#=================================================================================================
# Create a help context which refelct the status of the <Control>, <Shift> and <Alt> keys
#
#		helpCreate widget variable help ?track?
#
#			widget = which widget to associate with the help
#			variable = variable which will change according to the key pressed
#			help = an array containing the help text for the value 
#				   'normal', 'shift', control' and 'alt'
#			track = a variable which tracks the status of key which takes the values
#					'normal', 'shift', control' or 'alt'


proc helpCreate {w var help {track {}}} {
	bind $w <Enter> "+focus $w; set $var \[set ${help}(normal)\]"
	bind $w <KeyPress-Shift_L> "+set $var \[set ${help}(shift)\]"
	bind $w <KeyRelease-Shift_L> "+set $var \[set ${help}(normal)\]"
	bind $w <KeyPress-Shift_R> "+set $var \[set ${help}(shift)\]"
	bind $w <KeyRelease-Shift_R> "+set $var \[set ${help}(normal)\]"
	bind $w <KeyPress-Control_L> "+set $var \[set ${help}(control)\]"
	bind $w <KeyRelease-Control_L> "+set $var \[set ${help}(normal)\]"
	bind $w <KeyPress-Control_R> "+set $var \[set ${help}(control)\]"
	bind $w <KeyRelease-Control_R> "+set $var \[set ${help}(normal)\]"
	bind $w <KeyPress-Alt_L> "+set $var \[set ${help}(alt)\]"
	bind $w <KeyRelease-Alt_L> "+set $var \[set ${help}(normal)\]"
	bind $w <Leave> "+set $var \[set ${help}(off)\]"

	if {$track != {}} {
		bind $w <KeyPress-Shift_L> "+set $track shift"
		bind $w <KeyRelease-Shift_L> "+set $track normal"
		bind $w <KeyPress-Shift_R> "+set $track shift"
		bind $w <KeyRelease-Shift_R> "+set $track normal"
		bind $w <KeyPress-Control_L> "+set $track control"
		bind $w <KeyRelease-Control_L> "+set $track normal"
		bind $w <KeyPress-Control_R> "+set $track control"
		bind $w <KeyRelease-Control_R> "+set $track normal"
		bind $w <KeyPress-Alt_L> "+set $track alt"
		bind $w <KeyRelease-Alt_L> "+set $track normal"
		set $track normal
	}
}

#=================================================================================================

proc checkNumber {w value} {
	if {[scan $value "%f" dummy] != 1} {
		return 0;
	}
	return 1
}

#=================================================================================================

proc stringify {str} {
	regsub -all " " $str "_" result
	set result [string tolower $result]
	return $result
}

#=================================================================================================

proc replaceBindtags {w old new} {
	set list [bindtags $w]
	set i [lsearch -exact $list $old]
	bindtags $w [lreplace $list $i $i $new]
}




#=================================================================================================
#	Manage a tip ballon
#
#		registerTip text widget ?id? ?callback?
#
#				text = what to display
#				widget = what widget is displaying
#				id = id for an item of a canvas
#				callback = procedure to call to generate the text to display

set __tip__(v:serial) 1

proc registerTip {text w {id ""} {callback {}}} {
global __tip__

	# if id given, then for a canvas, else a widget
	if {$id == ""} {
		bind $w <Enter> "+registerTip:display $__tip__(v:serial) %x %y"
		bind $w <Leave> "+registerTip:undisplay $__tip__(v:serial)"
	} else {
		$w bind $id <Enter> "+registerTip:display $__tip__(v:serial) %x %y"
		$w bind $id <Leave> "+registerTip:undisplay $__tip__(v:serial)"
	}
	set __tip__(txt:$__tip__(v:serial)) $text
	if {$text == ""} {
		set __tip__(cb:$__tip__(v:serial)) $callback
	}

	incr __tip__(v:serial)
}

proc registerSlowTip {text w {id ""} {callback {}}} {
global __tip__

	# if id given, then for a canvas, else a widget
	if {$id == ""} {
		bind $w <Enter> "+registerTip:display -$__tip__(v:serial) %x %y"
		bind $w <Leave> "+registerTip:undisplay $__tip__(v:serial)"
	} else {
		$w bind $id <Enter> "+registerTip:display -$__tip__(v:serial) %x %y"
		$w bind $id <Leave> "+registerTip:undisplay $__tip__(v:serial)"
	}
	set __tip__(txt:$__tip__(v:serial)) $text
	if {$text == ""} {
		set __tip__(cb:$__tip__(v:serial)) $callback
	}

	incr __tip__(v:serial)
}

#-------------------------------------------------------------------------------------------------

proc registerTip:display {serial x y} {
global __tip__ status

	if {$serial < 0} {
		set serial [expr -$serial]
		set __tip__(t:$serial) [after $status(v:tip:delayLong) \
				"registerTip:doIt $serial $x $y"]
	} else {
		set __tip__(t:$serial) [after $status(v:tip:delay) "registerTip:doIt $serial $x $y"]
	}

	# after 5 seconds kill it
	set __tip__(v:destroy) [after 5000 "destroy .tip"]
}

proc registerTip:doIt {serial x y} {
global __tip__ status

	if {$status(v:tip:show)} {
		destroy .tip
		toplevel .tip
		wm overrideredirect .tip 1
		wm geometry .tip +[expr [winfo pointerx .]+13]+[expr [winfo pointery .]+13]

		frame .tip.b -bg black -bd 1
		set text $__tip__(txt:$serial)
		if {$text == ""} {
			regsub -all "%x" $__tip__(cb:$serial) $x cb
			regsub -all "%y" $cb $y cb
			set text [uplevel #0 $cb]
		}
		label .tip.b.l -justify left -bg lightyellow -font tip -text $text
		pack .tip.b .tip.b.l
	}
}

proc registerTip:undisplay {serial} {
global __tip__

	catch {after cancel $__tip__(t:$serial)}
	catch {after cancel $__tip__(v:destroy)}
	destroy .tip
}



#=================================================================================================
#	Manage a basic pane windows
#

namespace eval pane {} {
}

proc pane::new {w left rsz right} {
variable pane

	set pane($w:left) $left
	set pane($w:rsz) $rsz
	set pane($w:right) $right

	bind $rsz <ButtonPress-1> "+pane::setup $w %x"
	bind $rsz <Button1-Motion> "+pane::move $w %x"
	bind $rsz <ButtonRelease-1> "+pane::doIt $w %x"
}

proc pane::setup {w x} {
variable pane

	set pane($w:x) [winfo rootx $pane($w:rsz)]
	set pane($w:y) [winfo rooty $pane($w:rsz)]
	set pane($w:h) [winfo height $pane($w:rsz)]
	set pane($w:oldx) $x
	tixTmpLine [expr $pane($w:x)+$x] $pane($w:y) \
			   [expr $pane($w:x)+$x] [expr $pane($w:y)+$pane($w:h)] $w
}

proc pane::move {w x} {
variable pane

	tixTmpLine [expr $pane($w:x)+$pane($w:oldx)] $pane($w:y) \
			   [expr $pane($w:x)+$pane($w:oldx)] [expr $pane($w:y)+$pane($w:h)] $w

	set width [expr [winfo width $pane($w:left)]+$x]
	if {$width<100} {
		set x [expr 100-[winfo width $pane($w:left)]]
	}
	set width [expr [winfo width $pane($w:right)]-4-$x]
	if {$width<100} {
		set x [expr [winfo width $pane($w:right)]-104]
	}
	tixTmpLine [expr $pane($w:x)+$x] $pane($w:y) \
			   [expr $pane($w:x)+$x] [expr $pane($w:y)+$pane($w:h)] $w
	set pane($w:oldx) $x
}

proc pane::doIt {w x} {
variable pane

	tixTmpLine [expr $pane($w:x)+$pane($w:oldx)] $pane($w:y) \
			   [expr $pane($w:x)+$pane($w:oldx)] [expr $pane($w:y)+$pane($w:h)] $w

	$pane($w:left) configure -width [expr [winfo width $pane($w:left)]+$pane($w:oldx)]
	$pane($w:right) configure -width [expr [winfo width $pane($w:right)]-4-$pane($w:oldx)]
}



#=================================================================================================
#	Simple dialog to ask a string
#

proc getStringFromDialog {title text label initvalue varname} {
global __getstringstatus__ __getstringvalue__
upvar 1 $varname result

	set top .__getstring__

	# create the window
    toplevel $top
    wm title $top $title
    bind $top <Alt-w> "set __getstringstatus__ 0"
    bind $top <Alt-o> "set __getstringstatus__ 1"
    bind $top <Alt-c> "set __getstringstatus__ 0"
	bind $top <Return> "set __getstringstatus__ 1"
    bind $top <Escape> "set __getstringstatus__ 0"

	# edition of the description
	label $top.text -text $text

	# edition of the string
	label $top.string -text $label -underline 0
	set __getstringvalue__ $initvalue
	entry $top.stringEdit -textvariable __getstringvalue__

	# the buttons
	set f [frame $top.buttons]
	  button $f.ok -default active -text "OK" -width 5 -underline 0 \
			-command "set __getstringstatus__ 1" 
	  button $f.cancel -text "Cancel" -command "set __getstringstatus__ 0" -width 5 -underline 0
	  pack $f.ok $f.cancel -side left -padx 10

	# the layout
	grid $top.text		-				-sticky new -padx 5 -pady 5
	grid $top.string	$top.stringEdit	-sticky new -padx 5 -pady 5
	grid $top.buttons	-				-sticky ns -padx 5 -pady 5
	grid rowconfig    $top {0 2} -weight 0
	grid rowconfig    $top {1} -weight 1
	grid columnconfig $top {0} -weight 0
	grid columnconfig $top {1} -weight 1

	# give the focus
	grab set $top
	focus $top.stringEdit
	tkwait variable __getstringstatus__
	grab release $top
	destroy $top

	if {$__getstringstatus__} {
		set result $__getstringvalue__
		return 1
	} else {
		return 0
	}
}


#=================================================================================================
#	Simple dialog to ask a string
#

proc buildModalDialog {title} {
global __waiting__

	# create the window
    toplevel .__dialog__
    wm title .__dialog__ $title
    bind .__dialog__ <Alt-w> "set __waiting__ 1"
    bind .__dialog__ <Alt-o> "set __waiting__ 1"
	bind .__dialog__ <Return> "set __waiting__ 1"
    bind .__dialog__ <Escape> "set __waiting__ 1"
	frame .__dialog__.frame
	return .__dialog__.frame
}

proc waitforModalDialog {} {
	button .__dialog__.ok -default active -text "OK" \
			-width 5 -underline 0 -command "set __waiting__ 1"
	
	pack .__dialog__.frame -padx 10 -pady 5 -expand true -fill both
	pack .__dialog__.ok -padx 10 -pady 5
	grab set .__dialog__
	tkwait variable __waiting__
	grab release .__dialog__
	destroy .__dialog__
}


#=================================================================================================
# Rewrites tkMessageBox to have nicer images

image create pixmap dlg_error -file pixmap/dlg_error.xpm
image create pixmap dlg_info -file pixmap/dlg_info.xpm
image create pixmap dlg_question -file pixmap/dlg_question.xpm
image create pixmap dlg_warning -file pixmap/dlg_warning.xpm


proc tkMessageBox {args} {
	global tkPriv tcl_platform

	set w tkPrivMsgBox
	upvar #0 $w data

	#
	# The default value of the title is space (" ") not the empty string
	# because for some window managers, a 
	#				wm title .foo ""
	# causes the window title to be "foo" instead of the empty string.
	#
	set specs {
		{-default "" "" ""}
		{-icon "" "" "info"}
		{-message "" "" ""}
		{-parent "" "" .}
		{-title "" "" " "}
		{-type "" "" "ok"}
	}

	tclParseConfigSpec $w $specs "" $args

	if {[lsearch {info warning error question} $data(-icon)] == -1} {
		error "invalid icon \"$data(-icon)\", must be error, info, question or warning"
	}

	if ![winfo exists $data(-parent)] {
		error "bad window path name \"$data(-parent)\""
	}

	case $data(-type) {
		abortretryignore {
			set buttons {
				{abort  -width 6 -text Abort -under 0}
				{retry  -width 6 -text Retry -under 0}
				{ignore -width 6 -text Ignore -under 0}
			}
		}
		ok {
			set buttons {
				{ok -width 6 -text OK -under 0}
			}
			if {$data(-default) == ""} {
				set data(-default) "ok"
			}
		}
		okcancel {
			set buttons {
				{ok	 -width 6 -text OK	 -under 0}
				{cancel -width 6 -text Cancel -under 0}
			}
		}
		retrycancel {
			set buttons {
				{retry  -width 6 -text Retry  -under 0}
				{cancel -width 6 -text Cancel -under 0}
			}
		}
		yesno {
			set buttons {
				{yes	-width 6 -text Yes -under 0}
				{no	 -width 6 -text No  -under 0}
			}
		}
		yesnocancel {
			set buttons {
				{yes	-width 6 -text Yes -under 0}
				{no	 -width 6 -text No  -under 0}
				{cancel -width 6 -text Cancel -under 0}
			}
		}
		default {
			error "invalid message box type \"$data(-type)\", must be abortretryignore, ok, okcancel, retrycancel, yesno or yesnocancel"
		}
	}

	if {[string compare $data(-default) ""]} {
		set valid 0
		foreach btn $buttons {
			if ![string compare [lindex $btn 0] $data(-default)] {
				set valid 1
				break
			}
		}
		if !$valid {
			error "invalid default button \"$data(-default)\""
		}
	}

	# 2. Set the dialog to be a child window of $parent
	#
	#
	if {[string compare $data(-parent) .]} {
		set w $data(-parent).__tk__messagebox
	} else {
		set w .__tk__messagebox
	}

	# 3. Create the top-level window and divide it into top
	# and bottom parts.

	catch {destroy $w}
	toplevel $w -class Dialog
	wm title $w $data(-title)
	wm iconname $w Dialog
	wm protocol $w WM_DELETE_WINDOW { }
	wm transient $w $data(-parent)
	if {$tcl_platform(platform) == "macintosh"} {
		unsupported1 style $w dBoxProc
	}

	frame $w.bot
	pack $w.bot -side bottom -fill both
	frame $w.top
	pack $w.top -side top -fill both -expand 1
	if {$tcl_platform(platform) != "macintosh"} {
		$w.bot configure -relief raised -bd 1
		$w.top configure -relief raised -bd 1
	}

	# 4. Fill the top part with bitmap and message (use the option
	# database for -wraplength so that it can be overridden by
	# the caller).

	option add *Dialog.msg.wrapLength 3i widgetDefault
	label $w.msg -justify left -text $data(-message)
	catch {$w.msg configure -font \
				-Adobe-Times-Medium-R-Normal--*-180-*-*-*-*-*-*
	}
	pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
	if {$data(-icon) != ""} {
		label $w.bitmap -image dlg_$data(-icon)
		pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
	}

	# 5. Create a row of buttons at the bottom of the dialog.

	set i 0
	foreach but $buttons {
		set name [lindex $but 0]
		set opts [lrange $but 1 end]
		if ![string compare $opts {}] {
			# Capitalize the first letter of $name
			set capName \
				[string toupper \
					[string index $name 0]][string range $name 1 end]
			set opts [list -text $capName]
		}

		eval button $w.$name $opts -command [list "set tkPriv(button) $name"]

		if ![string compare $name $data(-default)] {
			$w.$name configure -default active
		}
		pack $w.$name -in $w.bot -side left -expand 1 \
			-padx 3m -pady 2m

		# create the binding for the key accelerator, based on the underline
		#
		set underIdx [$w.$name cget -under]
		if {$underIdx >= 0} {
			set key [string index [$w.$name cget -text] $underIdx]
			bind $w <Alt-[string tolower $key]>  "$w.$name invoke"
			bind $w <Alt-[string toupper $key]>  "$w.$name invoke"
		}
		incr i
	}

	# 6. Create a binding for <Return> on the dialog if there is a
	# default button.

	if {[string compare $data(-default) ""]} {
		bind $w <Return> "tkButtonInvoke $w.$data(-default)"
	}

	# 7. Withdraw the window, then update all the geometry information
	# so we know how big it wants to be, then center the window in the
	# display and de-iconify it.

	wm withdraw $w
	update idletasks
	set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
			- [winfo vrootx [winfo parent $w]]]
	set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
			- [winfo vrooty [winfo parent $w]]]
	wm geom $w +$x+$y
	wm deiconify $w

	# 8. Set a grab and claim the focus too.

	set oldFocus [focus]
	set oldGrab [grab current $w]
	if {$oldGrab != ""} {
		set grabStatus [grab status $oldGrab]
	}
	grab $w
	if {[string compare $data(-default) ""]} {
		focus $w.$data(-default)
	} else {
		focus $w
	}

	# 9. Wait for the user to respond, then restore the focus and
	# return the index of the selected button.  Restore the focus
	# before deleting the window, since otherwise the window manager
	# may take the focus away so we can't redirect it.  Finally,
	# restore any grab that was in effect.

	tkwait variable tkPriv(button)
	catch {focus $oldFocus}
	destroy $w
	if {$oldGrab != ""} {
		if {$grabStatus == "global"} {
			grab -global $oldGrab
		} else {
			grab $oldGrab
		}
	}
	return $tkPriv(button)
}
