#-------------------------------------------------------------------------------------------------
#
# The concept : you have to register the 'source' and 'target' for the drag&drop.
#				'source' is a widget able to export (send) datas
#				'target' is a widget able to import (receive) datas
#
#-------------------------------------------------------------------------------------------------
#
# Options for SOURCE :
#		-button			1 or {Control 2} (default 1)
#		-type			type of the source (default any)
#		-send			what to send (if no check)
#		-checkcmd		routine to check if source ok (arg = w x y) return what to send
#		-cursor			cursor to use when dragging
#
# Options for TARGET :
#		-type			list of the type accepted or any (any by default)
#		-cmd			command to call (appended arg = widget type sentValue)
#		-checkcmd		routine to check if target ok inside the widget (arg = w x y sentValue)
#						return the cursor to use to show acceptance
#		-cursor			cursor to use to show acceptance
#		-cleanupcmd		proc called when the cursor leaves the windows
#		-releasecmd		proc called after release
#
#-------------------------------------------------------------------------------------------------

namespace eval drag&drop {
}

proc drag&drop::source {cmd args} {
variable data

	set w [lindex $args 0]

	switch -exact $cmd {
		add		  {	lappend data(v:sources) $w }
		delete	  {	set i [lsearch -exact $data(v:sources) $w]
					set data(v:sources) [lreplace $data(v:sources) $i $i]
					foreach v [array names data(tgt:$w:*)] { unset $v }
					#TODO : remove bindings
				  }
		list	  {	return $data(v:sources) }
	}

	# defaults
	set button 1
	set data(src:$w:send) ""
	set data(src:$w:type) "any"
	set data(src:$w:cursor) "fleur"

	# parse options
	foreach {option value} [lrange $args 1 end] {
		switch -exact -- $option {
			-button		{ set button $value }
			-checkcmd	{ set data(src:$w:check) $value }
			-send		{ set data(src:$w:send) $value }
			-type		{ set data(src:$w:type) $value }
			-cursor		{ set data(src:$w:cursor) $value }
		}
	}

	# register the bindings
	if {[llength $button]>1} {
		set m [lindex $button 0]
		set b [lindex $button 1]
		bind $w <$m-ButtonPress-$b> "+drag&drop::init %W %x %y %X %Y"
		bind $w <$m-Button$b-Motion> "+drag&drop::drag %W %X %Y"
		bind $w <$m-ButtonRelease-$b> "+drag&drop::drop %W %X %Y"
	} else {
		bind $w <ButtonPress-$button> "+drag&drop::init %W %x %y %X %Y"
		bind $w <Button$button-Motion> "+drag&drop::drag %W %X %Y"
		bind $w <ButtonRelease-$button> "+drag&drop::drop %W %X %Y"		
	}
}

proc drag&drop::target {cmd args} {
variable data

	set w [lindex $args 0]

	switch -exact $cmd {
		add		  {	lappend data(v:targets) $w }
		delete	  {	set i [lsearch -exact $data(v:targets) $w]
					set data(v:sources) [lreplace $data(v:targets) $i $i] 
					foreach v [array names data(tgt:$w:*)] { unset $v } }
		list	  {	return $data(v:sources) }
	}

	# defaults
	set data(tgt:$w:cmd) ""
	set data(tgt:$w:type) "any"
	set data(tgt:$w:cursor) "circle"
	set data(tgt:$w:check) "drag&drop::dummy"

	# parse options
	foreach {option value} [lrange $args 1 end] {
		switch -exact -- $option {
			-cmd		{ set data(tgt:$w:cmd) $value }
			-type		{ set data(tgt:$w:type) $value }
			-cursor		{ set data(tgt:$w:cursor) $value }
			-checkcmd	{ set data(tgt:$w:check) $value }
			-cleanupcmd	{ set data(tgt:$w:cleanup) $value }
			-releasecmd	{ set data(tgt:$w:release) $value }
		}
	}
}

#-------------------------------------------------------------------------------------------------

proc drag&drop::init {w x y X Y} {
variable data

	if {[info exists data(src:$w:check)]} {
		set data(src:$w:send) [eval $data(src:$w:check) $w $x $y]
		if {$data(src:$w:send) == ""} {
			set data(t:good) 0
			return
		}
	}
	set data(t:target) {}
	set data(t:good) 1
	set data(t:ok) 0
	set data(t:cursor) [[winfo containing $X $Y] cget -cursor]
	$w configure -cursor $data(src:$w:cursor)
}

#-------------------------------------------------------------------------------------------------

proc drag&drop::drag {w X Y} {
variable data

	if !$data(t:good) {
		return
	}
	set wdest  [winfo containing $X $Y]
	if {$wdest == ""} {
		return
	}
	set top [winfo parent [winfo toplevel $wdest]]

	# if we are over a target : check from lowest window until we reach the toplevel
	while {$wdest != $top} {
		if {[info exists data(tgt:$wdest:type)]} {
			if {$data(tgt:$wdest:type) == "any"
				|| [lsearch -exact $data(tgt:$wdest:type) $data(src:$w:type)] != -1} {

				set x [expr [winfo pointerx $wdest]-[winfo rootx $wdest]]
				set y [expr [winfo pointery $wdest]-[winfo rooty $wdest]]
				set cursor [eval $data(tgt:$wdest:check) $wdest $x $y [list $data(src:$w:send)]]
				if {$cursor != ""} {
					$w configure -cursor $cursor
					set data(t:ok) 1
					set data(t:target) $wdest
					return
				}
			}
		}
		# try the parent window
		set wdest [winfo parent $wdest]
	}

	$w configure -cursor $data(src:$w:cursor)
	set data(t:ok) 0
	if {$data(t:target) != {} && [info exists data(tgt:$data(t:target):cleanup)]} {
		eval $data(tgt:$data(t:target):cleanup)
	}
	set data(t:target) {}
}

#-------------------------------------------------------------------------------------------------

proc drag&drop::drop {w X Y} {
variable data

	if {$data(t:good)} {
		$w configure -cursor $data(t:cursor)
		if {$data(t:ok)} {
			set wdest $data(t:target)
			eval $data(tgt:$wdest:cmd) $wdest $data(src:$w:type) [list $data(src:$w:send)]
		}
		if {$data(t:target) != {} && [info exists data(tgt:$data(t:target):cleanup)]} {
			eval $data(tgt:$data(t:target):cleanup)
		}
	}
	if {[info exists data(t:target)] && [info exists data(tgt:$data(t:target):release)]} {
		eval $data(tgt:$data(t:target):release)
	}	
}

#-------------------------------------------------------------------------------------------------

proc drag&drop::dummy {w args} {
variable data

	return $data(tgt:$w:cursor)
}


##################################################################################################


proc dummy {} {
global id
global i
global c

	foreach w [winfo children .] { destroy $w }

	toplevel .t
	wm geometry .t +0+300
	canvas .t.c
	bind .t.c <Button2-Motion> "puts \"%x -- %y\""
	set id(1) [.t.c create rectangle 100 100 200 200 -fill red]
	set id(2) [.t.c create rectangle 50 150 0 100 -fill green]
	pack .t.c

	proc test {w x y sent} {
	global id

		if {[$w find overlapping $x $y $x $y] == $id(1)} {
			return "target"
		} elseif {[$w find overlapping $x $y $x $y] == $id(2)} {
			return "coffee_mug"
		}
		return ""
	}

	proc test2 {w x y} {
	global id

		if {[$w find overlapping $x $y $x $y] == $id(1)} {
			return "red"
		} elseif {[$w find overlapping $x $y $x $y] == $id(2)} {
			return "green"
		}
		return ""
	}


	label .l -text "coucou" -bg orange
	pack .l

	catch {unset drag&drop::data}
	set drag&drop::data(t:old:w) "list"
	set drag&drop::data(t:old:cursor) ""

	drag&drop::source add .l -send "coucou" -type "toto" -cursor diamond_cross
	drag&drop::target add .t.c -checkcmd "test" -cmd "puts " -type "toto" -cursor target

	drag&drop::source add .t.c -checkcmd "test2" -type "titi" -cursor fleur
	drag&drop::target add .l -cmd "puts " -type "titi" -cursor circle

	parray drag&drop::data


	set f [open ccccc]
	set i 0
	while ![eof $f] {
		gets $f toto
		scan $toto "#define XC_%s" c($i)
		incr i
	}

	set i 0
	bind . <Key> toto
	proc toto {} {
	global c
	global i
	
		puts $c($i)
		. configure -cursor $c($i)
		incr i
	}
}
