namespace eval library {
	variable data

	# creation of the body for the pose
	set data(v:body:internal) [body "__library1__"]
	$data(v:body:internal) select all
	set data(v:body:internal2) [body "__library2__"]
	set data(v:body:update) [body "Update of Library"]

	# creation of the window for the choice of view
	set data(w:pose:dialog) [toplevel .libraryDialog]
	  wm withdraw $data(w:pose:dialog)
	  wm title $data(w:pose:dialog) "Definition of the pose"
	  wm geometry $data(w:pose:dialog) +300+300
	# creation of the window for the choice of view
	set data(w:position:dialog) [toplevel .libraryDialog2]
	  wm withdraw $data(w:position:dialog)
	  wm title $data(w:position:dialog) "Definition of the position"
	  wm geometry $data(w:position:dialog) +300+300

	# some internal variables
	set data(v:pose:view:floor) 0
	set data(v:pose:axis) 0
	set data(v:pose:forcewire) 0

	set data(v:position:floor) 1
	set data(v:position:axis) 0
	set data(v:position:forcewire) 0
	set data(v:position:mode) top
	set data(v:position:3d) 	{-35  22   0	 5	  0 -.7}
	set data(v:position:top) 	{  0  90   0	 1    0   0}

	set data(v:name) ""
	set data(v:class) ""

	variable status
	variable category ""
}

#=================================================================================================
#
# The library
#

proc library::init {{pos ""} {size ""}} {
global status
variable list
variable data
variable categories

	set w [toplevel .library -takefocus true]
	if {$pos != "" || $size != ""} {
	    wm geometry $w $size$pos
	}
    wm title $w "Library"
	wm protocol $w WM_DELETE_WINDOW "main::closeWindow .library"
	wm withdraw $w
	bind $w <Escape> "after idle \"main::closeWindow $w\""

	# generate special bindings for (h|t)list
	init:bindings

    # Main window = paned windows
    tixPanedWindow $w.top -paneborderwidth 0 -separatorbg gray50 -orient horizontal
    pack $w.top -side top -expand yes -fill both

	# the list box for the classes
	set ww [$w.top add class -min 50 -size 140]
	  tixScrolledHList $ww.class -scrollbar auto -highlightthickness 0
	  set data(w:class) [$ww.class subwidget hlist]
	  pack $ww.class -side top -expand yes -fill both -padx 3 -pady 2
	  # configure the hlist
	  $data(w:class) configure -browsecmd "library::class:select" \
			  -font library:class -selectmode browse -separator /
	  # fill the classes
	  foreach class [lsort [glob -nocomplain $status(dir:library)/*]] {
		  if {[file isdirectory $class]} {
			  set c [file tail $class]
			  $data(w:class) addchild {} -text $c
		  }
	  }
	  # action menu
	  bind $data(w:class) <3> "library::class:actionMenu %W %x %y %X %Y"
	  set m [menu $data(w:class).m -tearoff 0 -font menubutton \
					-bd 1 -activeborderwidth 1]
		$m add command -label "New class ..." -command "library::class:new"
		$m add command -label "Delete"
		bind $m <Unmap> "set tmp \[$data(w:class) info selection\]; \
						 if {\$tmp==\"\"} {$data(w:class) anchor clear} \
						 else {$data(w:class) anchor set \$tmp}"


	# the notebook
	set ww [$w.top add pose -min 50 -size 450 -expand 1]
	  set data(w:notebook) [tixNoteBook $ww.n]
	  $ww.n subwidget nbframe configure -bd 1 -font notebook
	  pack $ww.n -side top -expand yes -fill both -padx 3 -pady 2


	# the "Pose & Position" part
	foreach cat $categories {
		set entry [lindex $cat 0]
		set cmd "library::class:raisetab $w $entry"
	    set f [$ww.n add [stringify $entry] -label $entry -underline 0 -raisecmd $cmd]

		# the list with the snapshots
		tixScrolledTList $f.c -scrollbar "auto" -width 450 -height 140
		set data(w:tl:$entry) [$f.c subwidget tlist]
		$data(w:tl:$entry) configure -orient horizontal \
				-itemtype image -padx 3 -pady 3 -highlightcolor lightgray
		set data($entry:,name) ""	;# for motion over nothing
		bind $data(w:tl:$entry) <Motion> \
				"library::icons:browse \[[$f.c subwidget tlist] nearest %x %y\]"
		bind $data(w:tl:$entry) <Leave> "set library::status(comment) \"\";
										 set library::status(modif) \"\""
		replaceBindtags $data(w:tl:$entry) TixTList ReorderTList
		pack $f.c -side top -expand yes -fill both

		set list(config:$entry)  [lindex $cat 1]
	}
	menu $w.popup -tearoff 0 -font popupmenu -bd 1 -activeborderwidth 1
	  $w.popup add command -label "Edit ..."
	  $w.popup add command -label "Edit comment ..."
	  $w.popup add command -label "Delete"
	  $w.popup add separator
	  $w.popup add command -label "Copy"
	  $w.popup add command -label "Paste"


	# the "Routine" part
    set f [$ww.n add routine -label Routine -underline 0 \
				-raisecmd {set library::status(comment) ""; set library::status(modif) ""}]
	  # list box for category
	  set data(w:routine:treeclass) [tixTree $f.c -scrollbar "auto" -width 100 -height 140 \
				-highlightcolor lightgray -browsecmd "library::routine:category:load" \
				-options { hlist.indicator 0 hlist.itemType text hlist.header true hlist.padY 1}]
	  set data(w:routine:class) [$f.c subwidget hlist]
		set header [tixDisplayStyle text -refwindow $data(w:routine:class) \
				-fg black -anchor c -padx 8 -pady 2 -font library:routine:header]
		$data(w:routine:class) header create 0 -itemtype text -text Category \
				-style $header -borderwidth 1

		set data(v:style:left1) [tixDisplayStyle text -refwindow $data(w:routine:class) \
				-anchor w -font library:routine:text]
		set data(v:style:left1b) [tixDisplayStyle text -refwindow $data(w:routine:class) \
				-anchor w -font library:routine:header]
		bind $data(w:routine:class) <3> "library::routine:category:actionMenu %W %x %y %X %Y"
		set m [menu $data(w:routine:class).m -tearoff 0 -font menubutton \
					-bd 1 -activeborderwidth 1]
		  $m add command -label "New category ..."
		  $m add command -label "Delete"
		  bind $m <Unmap> "set tmp \[$data(w:routine:class) info selection\];
						   if {\$tmp==\"\"} {$data(w:routine:class) anchor clear} \
						   else {$data(w:routine:class) anchor set \$tmp}"
		# register as drop site for moving of routines
		drag&drop::target add $data(w:routine:class) -type Routine \
				-cmd "library::routine:move" -checkcmd "library::routine:move:check" \
				-releasecmd "set tmp \[$data(w:routine:class) info selection\]; \
							 if {\$tmp==\"\"} {$data(w:routine:class) anchor clear} \
							 else {$data(w:routine:class) anchor set \$tmp}"


	  # list (4 clomuns) to display info on routine
	  tixScrolledHList $f.l -highlightcolor lightgray -scrollbar "auto" -width 350 -height 140 \
			-options {hlist.columns 5
					  hlist.header  true }
	  set data(w:routine) [$f.l subwidget hlist]
	  bindtags $data(w:routine) [list $data(w:routine) DragHList .library all]

		set header [tixDisplayStyle text -refwindow $data(w:routine) \
				-fg black -anchor c -padx 8 -pady 2 -font library:routine:header]
		set lheader [tixDisplayStyle text -refwindow $data(w:routine) \
				-fg black -anchor w -padx 8 -pady 2 -font library:routine:header]

		$data(w:routine) header create 0 -itemtype text -text Name -style $header
		$data(w:routine) header create 1 -itemtype text -text Bts -style $header
		$data(w:routine) header create 2 -itemtype text -text Dcrs -style $header
		$data(w:routine) header create 3 -itemtype text -text Description -style $lheader
		$data(w:routine) header create 4
		foreach i {0 1 2 3 4} {
			$data(w:routine) header configure $i -borderwidth 1
		}
		$data(w:routine) column width 4 0
		# style for normal
		set data(v:style:normal:left) [tixDisplayStyle text -refwindow $data(w:routine) \
				-anchor nw -font library:routine:text]
		set data(v:style:normal:center) [tixDisplayStyle text -refwindow $data(w:routine) \
				-anchor n -font library:routine:text]
		set data(v:style:normal:right) [tixDisplayStyle text -refwindow $data(w:routine) \
				-anchor ne -font library:routine:text]
		# style for disabled
		set data(v:style:disabled:left) [tixDisplayStyle text -refwindow $data(w:routine) \
				-anchor nw -font library:routine:text -fg gray]
		set data(v:style:disabled:center) [tixDisplayStyle text -refwindow $data(w:routine) \
				-anchor n -font library:routine:text -fg gray]
		set data(v:style:disabled:right) [tixDisplayStyle text -refwindow $data(w:routine) \
				-anchor ne -font library:routine:text -fg gray]
		# action menu for selected routine
		bind $data(w:routine) <Double-1> "library::routine:doubleClick %W %x %y"
		bind $data(w:routine) <3> "library::routine:actionMenu %W %x %y %X %Y"
		set m [menu $data(w:routine).m -tearoff 0 -font menubutton \
					-bd 1 -activeborderwidth 1]
		  $m add command -label "Info ..."
		  $m add command -label "Edit ..."
		  $m add separator
		  $m add command -label "Delete"
		  bind $m <Unmap> "set tmp \[$data(w:routine) info selection\];
						   if {\$tmp==\"\"} {$data(w:routine) anchor clear} \
						   else {$data(w:routine) anchor set \$tmp}"

	  # pane the window between the lists
	  pane::new $w $f.c [frame $f.rsz -width 1 -cursor sb_h_double_arrow] $f.l
	  # pack the widgets
	  pack $f.c $f.rsz -side left -fill both
	  pack $f.l -side left -expand yes -fill both

	# select a page by default
   	$ww.n pageconfigure position -underline 1

	# bind the shortcuts
	bind $w <a> "$ww.n raise arm"
	bind $w <b> "$ww.n raise body"
	bind $w <f> "$ww.n raise foot"
	bind $w <l> "$ww.n raise leg"
	bind $w <p> "$ww.n raise pose"
	bind $w <o> "$ww.n raise position"
	bind $w <r> "$ww.n raise routine"

	# add a status line for displaying the name/comment
	frame $ww.s
	  label $ww.s.c -width 30 -relief sunken -bd 1 -font library:status \
			-textvariable library::status(comment)
	  label $ww.s.m -width 16 -relief sunken -bd 1 -font library:status \
			-textvariable library::status(modif)
	  pack $ww.s.c -side left -expand yes -fill x -padx 1
	  pack $ww.s.m -side left -padx 1 -anchor e
	pack $ww.s -fill x -pady 2 -padx 2
	set library::status(comment) ""
	set library::status(modif) ""

	# the drag & drop facility
	foreach cat $categories {
		set entry [lindex $cat 0]
		if {$entry == "Position"} {
			set type position
		} else {
			set type pose
		}
		drag&drop::source add $data(w:tl:$entry) -type $type -cursor cross \
				-checkcmd "library::drag:check:pose $entry"
	}
	drag&drop::source add $data(w:routine) -type Routine -cursor cross \
			-checkcmd "library::drag:check:routine"

	# some init
	set data(v:updateInProgress) 0
}

#-------------------------------------------------------------------------------------------------

proc library::init:bindings {} {
	# DragHList
    tixBind DragHList <ButtonPress-1> { library::routine:handle %W %y }

    tixBind DragHList <Up> { tixHList:UpDown %W prev "" }
    tixBind DragHList <Down> { tixHList:UpDown %W next "" }
    tixBind DragHList <Shift-Up> { tixHList:UpDown %W prev s }
    tixBind DragHList <Shift-Down> { tixHList:UpDown %W next s }
    tixBind DragHList <Left> { tixHList:LeftRight %W left }
    tixBind DragHList <Right> { tixHList:LeftRight %W right }
    tixBind DragHList <Prior> { %W yview scroll -1 pages }
    tixBind DragHList <Next> { %W yview scroll 1 pages }
    tixBind DragHList <Return> { tixHList:Keyboard-Activate %W }
    tixBind DragHList <space> { tixHList:Keyboard-Browse %W }

	# ReorderTList
    tixBind ReorderTList <ButtonPress-1> { set library::data(t:reorder:where) in;
										   library::reorder:press %W %x %y }
    tixBind ReorderTList <Button1-Motion> { library::reorder:motion %W %x %y }
    tixBind ReorderTList <ButtonRelease-1> { tkCancelRepeat; library::reorder:release %W %x %y }
    tixBind ReorderTList <Button1-Leave> { set library::data(t:reorder:where) out;
						set tkPriv(x) %x; set tkPriv(y) %y; set tkPriv(X) %X; set tkPriv(Y) %Y
						set tkPriv(afterId) [after 50 library::reorder:scan %W] }
    tixBind ReorderTList <Button1-Enter> { set library::data(t:reorder:where) in; tkCancelRepeat }

    tixBind ReorderTList <Double-ButtonPress-1> { library::icons:command %W %x %y }

    tixBind ReorderTList <3> { library::reorder:actionMenu .library.popup %W %x %y %X %Y }
}

#-------------------------------------------------------------------------------------------------

proc library::drag:check:pose {category w x y} {
variable data

	set id [$w nearest $x $y]
	if {$id == ""} {
		return ""
	} else {
		return $data($category:$id,pose)
	}
}

proc library::drag:check:routine {w x y} {
variable data

	set entry [$w nearest $y]
	if {$entry == "" || [$w entrycget $entry -state] == "disabled"} {
		return ""
	} else {
		return [list [$data(w:routine) info data $entry] \
					 [expr [$data(w:routine) item cget $entry 1 -text]*1000] \
					 $entry]		;# for use in the routine:move
	}
}


#=================================================================================================
#
#	Management of the classes of library
#

proc library::class:select {entry} {
variable categories
variable data


	set end 0
	catch { if {[string compare [tixEvent type] "<ButtonRelease-1>"]==0} { set end 1 } }

	if {$end} {
		return
	}

	freeImages
	foreach cat $categories {
		$data(w:tl:[lindex $cat 0]) delete 0 end
	}

	set data(v:class) [$data(w:class) entrycget $entry -text]
	class:load
}

proc library::class:invoke {name} {
variable data

	set entry ""
	foreach c [$data(w:class) info children] {
		if {[$data(w:class) entrycget $c -text] == $name} {
			set entry $c
			break
		}
	}
	if {$entry == ""} {
		return -code error "no class named $name in the library"
	}
	$data(w:class) anchor set $entry
	$data(w:class) selection clear
	$data(w:class) selection set $entry
	class:select $entry
}


#-------------------------------------------------------------------------------------------------

proc library::class:load {} {
global status
variable category
variable categories
variable data


	# we want to add
	set data(v:useID) -1

	#------------------------------------------------
	# indicate the need for refreshing pose & postion
	foreach cat $categories {
		set data(v:uptodate:[lindex $cat 0]) 0
	}
	if {$category == ""} {
		set category "Arm"
	}
	class:raisetab .library $category

	#----------------------
	# read the routine part
	library::routine:category:setlist
}

proc library::class:raisetab {w entry} {
variable categories
variable data
variable category
variable status

	# some common setting
	set category $entry
	set status(comment) ""
	set status(modif) ""

	# check if need refresh
	if {$entry == "Routine" || $data(v:class) == "" || $data(v:uptodate:$entry)} {
		return
	}
	# do refresh -- load the icons
	set data(v:uptodate:$entry) 1
	# get the list of files
	global status
	set in [open $status(dir:library)/$data(v:class)/$category/index "r"]
	set data($category:filelist) [gets $in]
	close $in

	foreach f $data($category:filelist) {
		# read the data
		set f $status(dir:library)/$data(v:class)/$category/$f
		set in [open $f "r"]
		set name [string range [gets $in] 1 end]
		set pose [string range [gets $in] 1 end]
		close $in

		set id [icons:addEntry $category $pose $name]
		icons:setIcon $category $id $f
	}
}

#-------------------------------------------------------------------------------------------------

proc library::class:actionMenu {w x y X Y} {
variable data

	set entry [$w nearest $y]
	if {$entry != ""} {
		$w anchor set $entry
	}

	if {$entry == "0" || $entry == ""} {
		$w.m entryconfigure 1 -state disabled
	} else {
		$w.m entryconfigure 1 -state normal \
				-command "library::class:delete $w [list $entry]"
	}

	tk_popup $w.m $X $Y
}

#-------------------------------------------------------------------------------------------------

proc library::class:new {} {
global status
variable data
variable categories


	if {[getStringFromDialog "Library" "Enter the name for the new class" Class "" class]} {
		# check the class
		if {$class == ""} {
			return
		}
		if ![string match {[-a-zA-Z0-9_. ]*} $class] {
			tk_messageBox -icon "error" -type ok -title "Error" \
					-message "Name should only contains letters, digits, space, dot, - and _"
			return
		}
		if {[file exists $status(dir:library)/$class]} {
			tk_messageBox -icon "error" -type ok -title "Error" \
					-message "The name '$class' is already used."
			return
		}
		# generate new files
		file mkdir $status(dir:routine)/$class
		foreach cat $categories {
			set cat [lindex $cat 0]
			file mkdir $status(dir:library)/$class/$cat
			close [open $status(dir:library)/$class/$cat/index "w"]
		}
		# update list
		$data(w:class) delete all
		foreach cl [lsort [glob -nocomplain $status(dir:library)/*]] {
			if {[file isdirectory $cl]} {
				set c [file tail $cl]
				if {$c == $class} {
					set entry [$data(w:class) addchild {} -text $c]
				} else {
					$data(w:class) addchild {} -text $c
				}
			}
		}
		class:select $class
		$data(w:class) selection clear
		$data(w:class) selection set $entry
		$data(w:class) anchor set $entry
	}
}

proc library::class:delete {w entry} {
global status
variable data


	routine:check:class $entry

	if {[llength $data(t:list)] > 0} {
		routine:check:showconflict "class"

	} else {
		if {[tk_messageBox -icon "question" -type yesno -default yes -title "Error" \
						-message "Do you really want to delete this class ?"] == "no"} {
			return
		}

		# do it
		file delete -force -- $status(dir:routine)/$entry
		file delete -force -- $status(dir:library)/$entry
		$w delete entry $entry
	}
}


#=================================================================================================
#
#	Management of the reordering of the TList with icons
#

proc library::reorder:press {w x y} {
variable data

	set entry [$w nearest $x $y]
	if {$entry == ""} {	return }
	$w selection clear
	$w selection set $entry
	$w anchor set $entry
	set data(t:reorder:start) $entry
}

proc library::reorder:motion {w x y} {
variable data

	set entry [$w nearest $x $y]
	if {$entry == ""} {	return }
	$w anchor set $entry
	icons:browse $entry
}

proc library::reorder:release {w x y} {
variable data
variable category


	$w anchor clear
	$w selection clear
	set entry [$w nearest $x $y]
	if {$entry == ""} {	return }
	if {$data(t:reorder:where) == "out" || $entry == $data(t:reorder:start)} {
		# if not in tlist or item is the same, do nothing
		return
	}

	# compute new index file
	set from [lindex $data($category:filelist) $data(t:reorder:start)]
	if {$entry < $data(t:reorder:start)} {
		set tmp [lreplace $data($category:filelist) $data(t:reorder:start) $data(t:reorder:start)]
		set tmp [linsert $tmp $entry $from]
	} else {
		set tmp [linsert $data($category:filelist) $entry $from]
		set tmp [lreplace $tmp $data(t:reorder:start) $data(t:reorder:start)]
	}

	reorder:newlist $tmp
}

proc library::reorder:scan {w} {
global tkPriv

    set x $tkPriv(x)
    set y $tkPriv(y)
    set X $tkPriv(X)
    set Y $tkPriv(Y)

    set out 0
    if {$y >= [winfo height $w]} {
		$w yview scroll 1 units
		set out 1
    }
    if {$y < 0} {
		$w yview scroll -1 units
		set out 1
    }
    if {$out} {
		set tkPriv(afterId) [after 50 library::reorder:scan $w]
    }
}

#-------------------------------------------------------------------------------------------------

proc library::reorder:actionMenu {menu w x y X Y} {
variable data

	set entry [$w nearest $x $y]
	if {$entry == ""} {
		return
	}
	$menu entryconfigure 0 -command "library::icons:command $w $x $y"
	$menu entryconfigure 1 -command "library::reorder:newcomment $w $entry"
	$menu entryconfigure 2 -command "library::reorder:delete $w $entry"
	$menu entryconfigure 4 -state disabled
	$menu entryconfigure 5 -state disabled

	tk_popup $menu $X $Y
}

proc library::reorder:delete {w entry} {
global status
variable data
variable category

	if {[tk_messageBox -icon "question" -type yesno -default yes -title "Error" \
						-message "Do you really want to delete this entry ?"] == "no"} {
		return
	}

	# rm file
	set name [lindex $data($category:filelist) $entry]
	file delete $status(dir:library)/$data(v:class)/$category/$name
	# update index
	set tmp [lreplace $data($category:filelist) $entry $entry]
	reorder:newlist $tmp
}

proc library::reorder:duplicate {w entry} {
variable data

	#TODO : reorder duplicate
	notImplemented
}

proc library::reorder:newcomment {w entry} {
global status
variable data
variable category


	if {[getStringFromDialog "Library update" \
			"Enter the comment for the entry" Comment $data($category:$entry,name) comment]} {
		# set new comment
		set data($category:$entry,name) $comment
		# generate new filename
		set path $status(dir:library)/$data(v:class)/$category
		set id [clock format [clock seconds] -format "p%Y%b%d%H%M%S.ppm"]
		set filename $path/$id
		# copy data with new comment
		set in [open $path/[lindex $data($category:filelist) $entry] "r"]
		set out [open $filename "w"]
		puts $out "#$comment"
		fcopy $in $out
		close $in
		close $out
		# remove old file
		file delete $path/[lindex $data($category:filelist) $entry]
		# update filelist
		set data($category:filelist) [lreplace $data($category:filelist) $entry $entry $id]
		set out [open $path/index "w"]
		puts $out $data($category:filelist)
		close $out
	}

	icons:browse $entry
}

proc library::reorder:newlist {list} {
global status
variable data
variable category

	# write new index file
	set out [open $status(dir:library)/$data(v:class)/$category/index "w"]
	puts $out $list
	close $out
	# update TList
	freeImages $category
	$data(w:tl:$category) delete 0 end
	# we want to add
	set data(v:useID) -1
	library::class:load:category $category	
}



#=================================================================================================
#
# Management of icons
#

proc library::icons:addEntry {category pose name} {
variable data

	# add the icon
	if {$data(v:useID) == -1} {
		set id [$data(w:tl:$category) insert end]
	} else {
		set id $data(v:useID)
		image delete $data($category:$id,image)
	}

	set data($category:$id,name) $name
	set data($category:$id,pose) $pose

	return $id
}

proc library::icons:setIcon {category id filename} {
variable data

	set img [image create photo -file $filename]
	$data(w:tl:$category) entryconfigure $id -image $img	
	set data($category:$id,image) $img
}

#-------------------------------------------------------------------------------------------------

proc library::icons:browse {id} {
variable data
variable status
variable category


	if {$data(v:class) != ""} {
		set status(comment) $data($category:$id,name)
catch {	
		scan [lindex $data($category:filelist) $id] "p%4d%3s%2d%2d%2d" y m d h mm
		set status(modif) "$d $m $y, [format %02d:%02d $h $mm]"
}
	}
}


#=================================================================================================
#
# Misc.
#

array set library::convert {
	head			"Head 0"
	body			"body 1"
	torso			"torso 1.1"
	pelvis			"pelvis 1.2"
	left:arm		"left:Arm 2"
	left:shoulder	"left:arm 2.1"
	left:elbow		"left:forearm 2.2"
	left:wrist		"left:hand 2.3"
	left:leg		"left:Leg 4"
	left:hip		"left:thigh 4.1"
	left:knee:X		"left:leg 4.2"
	left:foot		"left:Foot 4.3"
	right:arm		"right:Arm 3"
	right:shoulder	"right:arm 3.1"
	right:elbow		"right:forearm 3.2"
	right:wrist		"right:hand 3.3"
	right:leg		"right:Leg 5"
	right:hip		"right:thigh 5.1"
	right:knee:X	"right:leg 5.2"
	right:foot		"right:Foot 5.3"
}

proc library::icons:command {w x y} {
variable data
variable convert
variable category


	set id [$w nearest $x $y]

	if {$category == "Position"} {
		# TODO : edition of position via library
		notImplemented
		return
	}

	if {$data(v:updateInProgress)} {
		tk_messageBox -icon "error" -title "Error" -type ok -default "ok" \
			-message "Already an update in progress"
		return
	}

	# prepare the body
	$data(v:body:update) set pose {{0.0 0.0 0.0} \
									 {{0.0 0.0 0.0} {0.0 0.0 0.0}} \
									 {{0.0 0.0 0.0 0.0} {0.0 0.0} {0.0 0.0 0.0}} \
									 {{0.0 0.0 0.0 0.0} {0.0 0.0} {0.0 0.0 0.0}} \
									 {{0.0 0.0 0.0} 0.0 {0.0 0.0 0.0} 0.0} \
									 {{0.0 0.0 0.0} 0.0 {0.0 0.0 0.0} 0.0} }
	$data(v:body:update) unselect all
	set data(t:parts) {}
	foreach entry $data($category:$id,pose) {
		set joint [lindex $entry 0]
		$data(v:body:update) set $joint [lindex $entry 1]
		$data(v:body:update) select [lindex $convert($joint) 0]
		lappend data(t:parts) [lindex $convert($joint) 1]
 	}

	# edit it
	figEditor::new .libraryUpdate {} $data(v:body:update)
	set data(v:updateInProgress) 1
	set data(t:category) $category
	set data(t:id) $id
}

proc library::callback:update {clientData} {
variable data

	set data(v:updateInProgress) 0
	if {$clientData == "cancel"} {
		return
	}

#puts "UPDATE  $data(t:category) -- $data(t:id) -- view = $clientData"
	library::pose:dialog update $clientData $data(v:body:update)
}


#-------------------------------------------------------------------------------------------------

proc library::callback:ok {w type} {
global status
variable list
variable data
variable category


	# generate the pose
	set pose {}
	if {$type == "position"} {
		lappend pose [list cog:X [$data(v:body:internal) get cog:X]]
		lappend pose [list cog:Z [$data(v:body:internal) get cog:Z]]
	}
	foreach sel [$data(w:$type:list) getselection] {
		set part [lindex $list($sel:joint) 0]
		lappend pose [list $part [$data(v:body:internal) get $part]]
	}

	# add/update the icon
	set entry [icons:addEntry $data(v:tab) $pose $data(v:name)]

	# save it
	set path $status(dir:library)/$data(v:class)/$data(v:tab)
	set id [clock format [clock seconds] -format "p%Y%b%d%H%M%S.ppm"]
	set filename $path/$id
	set out [open $filename "w"]
	fconfigure $out -translation binary
	puts $out "#$data(v:name)"
	puts $out "#$pose"
	puts $out [$data(w:$type:view) snapshot]
	flush $out
	close $out

	# remove old file
	set oldfile [lindex $data($category:filelist) $entry]
	if {$oldfile != ""} {
		file delete $path/$oldfile
	}

	# update the icon in the tabular list & filelist
	if {[info exists data($data(v:tab):$entry,image)]} {
		image delete $data($data(v:tab):$entry,image)
	}
	icons:setIcon $data(v:tab) $entry $filename
	if {$entry >= [llength $data($data(v:tab):filelist)]} {
		lappend data($data(v:tab):filelist) $id
	} else {
		set data($data(v:tab):filelist) [lreplace $data($data(v:tab):filelist) $entry $entry $id]
	}
	set out [open $status(dir:library)/$data(v:class)/$data(v:tab)/index "w"]
	puts $out $data($data(v:tab):filelist)
	close $out

	# "close" the dialog
	wm withdraw $w
}

#-------------------------------------------------------------------------------------------------

proc library::freeImages {{category {}}} {
variable data

	foreach img [array names data ${category}*image] {
		image delete $data($img)
		unset data($img)
	}
}


#=================================================================================================

# Categories : dependency with popup menu in figedit.tcl

set library::categories {
	{Arm		{2 3}}
	{Body		{0 1}}
	{Foot		{4.3 5.3}}
	{Leg		{4 5}}
	{Pose		{0 1 2 3 4 5}}
	{Position	{11 12 13 14}}
}
