#include "togl.h"

#include "list.h"
#include "view.h"

class List<Togl> *Light::views = new List<Togl>;
int Light::changed = TRUE;

//================================================================================================

Light::Light(int counter, char *n)
{
	number = counter;
	position[0] = position[1] = position[3] = 0.0; position[2] = 1.0;
	ambient[0] = ambient[1] = ambient[2] = ambient[3] = 0.0;
	diffuse[0] = diffuse[1] = diffuse[2] = diffuse[3] = 1.0;
	name = new char[strlen(n)+1];
	strcpy(name, n);
}

#include "light.icc"

//------------------------------------------------------------------------------------------------

int Light_ObjCmd(ClientData clientData, Tcl_Interp *interp,
			     int objc, Tcl_Obj *CONST objv[])
{
class Light *light = (class Light *) clientData;
static char *optionsPtr[] = {
	"set", "get", "remove", (char *) NULL};
enum {
	set, get, remove } index;


    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "option");
        return TCL_ERROR;
    }


    if (Tcl_GetIndexFromObj(interp, objv[1], optionsPtr, "option", 0, (int *) &index) != TCL_OK) {
        return TCL_ERROR;
    }

	if (index < remove && objc <3) {
        Tcl_WrongNumArgs(interp, 1, objv, "option arg ?arg ...?");
        return TCL_ERROR;
	}

	switch (index) {
		case set:		return light->Set(interp, objc, objv);
		case get:		return light->Get(interp, objc, objv);

		case remove:	View3D::lights->remove(light);
						// force the redisplay
						for(ListIterator<Togl> t(Light::views); t; ++t) {
							Togl_PostRedisplay(t());
						}
						light->changed = TRUE;

						char tmp[10];
						sprintf(tmp, "light%d", light->number);
						Tcl_DeleteCommand(interp, tmp);
						delete light;
						break;
	}

	return TCL_OK;
}


//------------------------------------------------------------------------------------------------

int LightCreate_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
static int counter = 0;
char cmd[10];
struct Light *l;

    if (argc < 2) {
        Tcl_SetResult(interp, "wrong # of arguments : light name", NULL);
        return TCL_ERROR;
    }

	// create the light
	l = new Light(counter, argv[1]);
	View3D::lights->append(l);
	// force the redisplay
	for(ListIterator<Togl> t(Light::views); t; ++t) {
		Togl_PostRedisplay(t());
	}

	// create the new command
	sprintf(cmd, "light%d", counter++);
	Tcl_CreateObjCommand(interp, cmd, Light_ObjCmd, 
					  (ClientData) l, (Tcl_CmdDeleteProc *) NULL);
	
	Tcl_SetResult(interp, cmd, TCL_VOLATILE);
	return TCL_OK;
}
