#include "list.h"

//------------------------------------------------------------------------------------------------
//		Double linked GenericLists (unique)
//------------------------------------------------------------------------------------------------

GenericList::~GenericList()
{
//fprintf(stderr, "%s  %08x\n", __PRETTY_FUNCTION__, this);
	while (d_head) {
		struct GenericListNode *toDie = d_head;
		d_head = d_head->d_next;
		delete toDie;
	}
}

//------------------------------------------------------------------------------------------------

void GenericList::append(void *obj)
{
struct GenericListNode *tmp;

	if (checkDuplicate(obj)) {
		return;
	}

	tmp = new GenericListNode(obj);
	tmp->d_prev = d_tail;
	if (d_tail == 0) {
		d_head = d_tail = tmp;
	} else {
		d_tail->d_next = tmp;
		d_tail = tmp;
	}
}

void GenericList::append(GenericList *list)
{
	list->d_head->d_prev = d_tail;
	if (d_tail == 0) {
		d_head = list->d_head;
		d_tail = list->d_tail;
	} else {
		d_tail->d_next = list->d_head;
		d_tail = list->d_tail;
	}
}

//------------------------------------------------------------------------------------------------

void GenericList::prepend(void *obj)
{
struct GenericListNode *tmp;

	if (checkDuplicate(obj)) {
		return;
	}

	tmp = new GenericListNode(obj);
	tmp->d_next = d_head;
	if (d_head == 0) {
		d_head = d_tail = tmp;
	} else {
		d_head->d_prev = tmp;
		d_head = tmp;
	}
}

void GenericList::prepend(GenericList *list)
{
	list->d_tail->d_next = d_head;
	if (d_head == 0) {
		d_head = list->d_head;
		d_tail = list->d_tail;
	} else {
		d_head->d_prev = list->d_tail;
		d_head = list->d_head;
	}
}

//------------------------------------------------------------------------------------------------

void GenericList::insert(GenericListNode *position, void *obj)
{
struct GenericListNode *tmp;

	if (checkDuplicate(obj)) {
		return;
	}

	tmp = new GenericListNode(obj);
	if (position == 0) {
		d_head = d_tail = tmp;
	} else {
		tmp->d_next = position->d_next;
		tmp->d_prev = position;
		position->d_next = tmp;
		if (tmp->d_next != 0) {
			tmp->d_next->d_prev = tmp;
		} else {
			d_tail = tmp; // update the d_tail
		}
	}
}

void GenericList::insert(GenericListNode *position, GenericList *list)
{
	if (position == 0) {
		d_head = list->d_head;
		d_tail = list->d_tail;
	} else {
		list->d_tail->d_next = position->d_next;
		list->d_head->d_prev = position;
		position->d_next = list->d_head;
		if (list->d_tail->d_next != 0) {
			list->d_tail->d_next->d_prev = list->d_tail;
		} else {
			d_tail = list->d_tail; // update the d_tail
		}
	}
}

//------------------------------------------------------------------------------------------------

void *GenericList::replace(void *oldobj, void *newobj)
{
struct GenericListNode *tmp;
void *data;

	for(tmp=d_head; tmp != 0; tmp=tmp->d_next) {
		if (tmp->d_data == oldobj) {
			data = tmp->d_data;
			tmp->d_data = newobj;
			return data;
		}
	}

	return 0;
}

//------------------------------------------------------------------------------------------------

void *GenericList::destroy(void *obj)
{
struct GenericListNode *tmp = d_head, *old = 0;
void *data;

	while (tmp != 0) {
		if (tmp->d_data == obj) {
			if (old == 0) {
				d_head = tmp->d_next;
				if (d_head) {
					d_head->d_prev = 0;
				} else {
					d_tail = 0;
				}
			} else {
				old->d_next = tmp->d_next;
				if (tmp->d_next != 0) {
					tmp->d_next->d_prev = old;
				} else {
					d_tail = old;
				}
			}
			tmp->d_next = 0;
			data = tmp->d_data;
			delete tmp;
			return data;
		}
		old = tmp; tmp = tmp->d_next;
	}
	return 0;
}

//================================================================================================

int GenericList::checkDuplicate(void *obj)
{
struct GenericListNode *tmp;

	for (tmp = d_head; tmp != 0; tmp = tmp->d_next) {
		if (tmp->d_data == obj)
			return -1;
	}
	return 0;
}

//================================================================================================

#include "togl.h"
#include "track.h"
template class List<Chunk>;
#include "routine.h"
template class List<Figure>;
template class List<Key>;
template class List<Modif>;
template class List<Note>;
template class List<TimeRange>;
#include "movie.h"
template class List<Bezier3D>;
template class List<Movie>;

//------------------------------------------------------------------------------------------------

template<class T> void List<T>::free()
{
	for(ListIterator<T> it(this); it; ++it)
		delete (T *) (it());
}

template<class T> void List<T>::replace(T *oldobj, T *newobj)
{
	delete (T *) GenericList::replace(oldobj,newobj);
}

template<class T> void List<T>::destroy(T *obj)
{
	delete (T *) GenericList::destroy(obj);
}
