namespace eval movie {
	variable list {}

	image create pixmap tbm_edit -file pixmap/tb_mv_edit.xpm
	image create pixmap tbm_view -file pixmap/tb_mv_view.xpm
	image create pixmap tbm_play -file pixmap/tb_mv_play.xpm
	image create pixmap tbm_record -file pixmap/tb_mv_record.xpm
	image create pixmap tbm_delete -file pixmap/tb_mv_delete.xpm
	image create pixmap tbm_height -file pixmap/tb_mv_height.xpm
	image create pixmap tbm_handle -file pixmap/tb_mv_handle.xpm
	image create pixmap tbm_paths -file pixmap/tb_mv_paths.xpm
	image create pixmap tbm_line -file pixmap/tb_mv_line.xpm
	image create pixmap tbm_notes -file pixmap/tb_mv_notes.xpm
}

#-------------------------------------------------------------------------------------------------

proc movie::new {w routineEditor {pos ""} {size ""}} {
global status tcl_platform
variable list
variable $w
upvar 0 $w data

    toplevel $w
	if {$pos != "" || $size != ""} {
	    wm geometry $w $size$pos
	}
	wm title $w "Movie <[[routine::getRoutine $routineEditor] get name]>"
	wm protocol $w WM_DELETE_WINDOW "main::closeWindow $w"
	bind $w <Escape> "after idle \"main::closeWindow $w\""

	# register the editor
	lappend list $w


	# the status bar
	set s [frame $w.s -height 16]
	label $s.help -relief sunken -bd 1 -font status \
			-textvariable movie::${w}(st:help)
	pack $s.help -side left -expand true -fill both
	pack propagate $s false


	# main frame
	set f [frame $w.top]
	  # the view
	  set data(w:view) [togl $f.t -width 300 -height 200 -ident "$w" \
				  -double true -depth true -class viewMovie -share $status(opengl)]
	  bind $data(w:view) <Motion> "movie::motion $w %x %y"
	  initView $w
	  helpCreate $data(w:view) movie::${w}(st:help) movie::help movie::${w}(v:keystatus)
	  bindtags $data(w:view) "[bindtags $data(w:view)] KeyMovie"

	  # notes
	  set data(w:notes) [frame $f.notes -width 160]

	  # the resize handle
	  pane::new $w $data(w:view) [frame $f.rsz -width 4 -cursor sb_h_double_arrow] $data(w:notes)

	pack $data(w:view) -side left -fill both -expand true -pady 2
	pack $f.rsz -side left -fill y
	pack $data(w:notes) -side left -fill both -expand true -pady 2
	pack propagate $f.t false
	pack propagate $f.notes false

	# the toolbar
	set data(w:tb) [toolbar::new $w.toolbar -bd 1]
	set tb $data(w:tb)
	  toolbar::add:radio $tb edit -bd 1 -group mode -image tbm_edit -tip "Edit mode" \
			-variable movie::${w}(v:mode) -oncommand "movie::set:mode $w"
	  toolbar::add:radio $tb view -bd 1 -group mode -image tbm_view -tip "View mode" \
			-variable movie::${w}(v:mode) -oncommand "movie::set:mode $w"
	  toolbar::add:radio $tb play -bd 1 -group mode -image tbm_play -tip "Play mode" \
			-variable movie::${w}(v:mode) -oncommand "movie::set:mode $w"
	  set data(v:old3D) [$data(w:view) get all]
	  toolbar::add:separator $tb 10 ;#----------------------------
	  toolbar::add:button $tb record -bd 1 -image tbm_record -tip "Record viewpoint" \
			-command "movie::record $w" -state disabled
	  toolbar::add:button $tb delete -bd 1 -image tbm_delete -tip "Delete this control point" \
			 -command "movie::delete $w" -state disabled
	  toolbar::add:separator $tb 10 ;#----------------------------
#	  toolbar::add:button $tb tobody -bd 1 -image tbm_record \
#			-tip "Set center in the middle of bodies" \
#			-command "movie::view:reset $w tobody" -state disabled
#	  toolbar::add:separator $tb 10 ;#----------------------------
	  toolbar::add:check $tb paths -bd 1 -image tbm_paths -tip "Show the control points paths" \
			-oncommand "$data(w:view) set showpaths 1" \
			-offcommand "$data(w:view) set showpaths 0" 
	  toolbar::add:check $tb height -bd 1 -image tbm_height -tip "Show height of control points" \
			-oncommand "$data(w:view) set showheight 1" \
			-offcommand "$data(w:view) set showheight 0" 
	  toolbar::add:check $tb handle -bd 1 -image tbm_handle -tip "Show handles of bezier curves" \
			-oncommand "$data(w:view) set showhandle 1" \
			-offcommand "$data(w:view) set showhandle 0" 
	  toolbar::add:check $tb line -bd 1 -image tbm_line -tip "Show line between eye and center" \
			-oncommand "$data(w:view) set showline 1" \
			-offcommand "$data(w:view) set showline 0" 
	  toolbar::add:separator $tb 10 ;#----------------------------
	  toolbar::add:check $tb notes -bd 1 -image tbm_notes -tip "Enable/disable notes" \
			-oncommand "movie::set:shownotes $w 1" \
			-offcommand "movie::set:shownotes $w 0" 

	  toolbar::vertical $tb
	  movie::set:shownotes $w 1
	  toolbar::check:invoke $tb paths

	#--------------------------------------------------------------------
	set fc [frame $w.control]
	# the player
	frame $fc.player
	player::new $fc.player "movie::sync:all $w" $data(w:view)
	player::setRoutine $fc.player [routine::getRoutine $routineEditor]
	bind $w <Key-space> "player::toggle $fc.player play"
	if {$tcl_platform(platform) == "windows"} {
		bind $w <Mod2-Key-space> "player::toggle $fc.player reverse"
	} else {
		bind $w <Mod1-Key-space> "player::toggle $fc.player reverse"
	}
	# list of movies for this routine
	menubutton $fc.movies -indicatoron 1 -menu $fc.movies.menu -width 20 \
			-font menubutton -textvariable movie::${w}(v:movie) \
			-relief raised -anchor w -direction below -bd 1
	set data(m:movies) [menu $fc.movies.menu -activeborderwidth 1 -bd 1 \
			-font menubutton -tearoff 0]
	# progress bar
	frame $fc.progress -bd 1 -relief sunken
	  set data(w:progress) [canvas $fc.progress.c -highlightthickness 0 -height 16 -width 100]
	  set data(v:progress:length) 1
	  set data(v:progress:id) [$fc.progress.c create rectangle 1 1 1 14]
	  pack $fc.progress.c -fill both -expand true
	  bind $fc.progress.c <Configure> "set movie::${w}(v:progress:length) %w; \
			movie::sync:all $w \[$data(w:view) get time\]"
	# the display of the time
	tixControl $fc.time -label "Time " -step 10 \
			-integer true -variable movie::${w}(v:time) -min 0 \
			-command "movie::sync:withcheck $w" \
			-options { label.font routine:time entry.font routine:time entry.justify right \
					   entry.borderWidth 1 entry.width 5 entry.highlightColor lightgray}
	# pack the controls
	pack $fc.movies -side left -padx 5 -fill x
	pack $fc.time   -side left -padx 10 -fill x
	pack $fc.player -side left -padx 5 -fill x
	pack $fc.progress -side left -padx 5 -fill x -expand true

	# the layout
	grid $data(w:tb)  $f	 	-sticky news -padx 2
	grid $w.control	  -			-sticky news -pady 2
	grid $w.s		  -		 	-sticky news -padx 1 -pady 1
	grid rowconfig    $w {0} -weight 1
	grid rowconfig    $w {1 2 3} -weight 0
	grid columnconfig $w {0} -weight 0
	grid columnconfig $w {1} -weight 1


	# the action menu for EDIT & VIEW
	menu $w.mae -tearoff 0 -font popupmenu -bd 1 -activeborderwidth 1
	  $w.mae add command -label "Align handles"
	  $w.mae add command -label "Linearize handles"
	  $w.mae add separator
	  $w.mae add command -label "Delete control point"
	menu $w.mav -tearoff 0 -font popupmenu -bd 1 -activeborderwidth 1
	  $w.mav add command -label "Same view as last" -command "movie::action:view $w view prev"
	  $w.mav add command -label "Same view as next" -command "movie::action:view $w view next"
	  $w.mav add separator
	  $w.mav add command -label "Same eye as last" -command "movie::action:view $w eye prev"
	  $w.mav add command -label "Same eye as next" -command "movie::action:view $w eye next"
	  $w.mav add separator
	  $w.mav add command -label "Same center as last" -command "movie::action:view $w center prev"
	  $w.mav add command -label "Same center as next" -command "movie::action:view $w center next"
	bind $data(w:view) <3> "movie::actionMenu $w %x %y %X %Y"


	# some settings
	set data(v:routineEditor) $routineEditor
	selectMovie $w [buildListMovies $w] 1	;# skip dialog if no movies defined

	set data(v:bodies) {}
	set routine [routine::getRoutine $routineEditor]
	set nb [$routine get dancers]
	for {set i -1} {$i < $nb} {incr i} {
		body:add $w $i
	}

	set movie::${w}(v:mode) edit
	toolbar::check:setOn $tb notes
	sync:all $w 0
}

proc movie::close {w} {
variable list
variable $w
upvar 0 $w data

	set i [lsearch -exact $list $w]
	set list [lreplace $list $i $i]
	# remove array associated with window
	catch {unset data}
}


#-------------------------------------------------------------------------------------------------

proc movie::body:add {w i} {
variable $w
upvar 0 $w data

	set routine [routine::getRoutine $data(v:routineEditor)]

	# create the frame for note
	if {$i == -1} {
		tixLabelFrame $data(w:notes).$i -padx 0 -pady 0 -label "Comments"
	} else {
		tixLabelFrame $data(w:notes).$i -padx 0 -pady 0 -label [$routine get dancername $i]
	}
	set data(w:notes:$i) [label [$data(w:notes).$i subwidget frame].l \
				-font movie:notes -justify left -anchor nw -height 5]
	pack $data(w:notes:$i) -padx 1 -pady 1 -expand true -fill both
	pack $data(w:notes).$i -expand true -fill both

	pack propagate [$data(w:notes).$i subwidget frame] false

	# record the body
	if {$i != -1} {
		set b [$routine get body $i]
		set newb [body [$b get name]]
		eval $newb setcolor [$b getcolor all]
		$newb set sex [$b get sex]
		$data(w:view) add $newb
		lappend data(v:bodies) $newb
	}
}

proc movie::body:get {w index} {
variable $w
upvar 0 $w data

	return [lindex $data(v:bodies) $index]
}

#-------------------------------------------------------------------------------------------------

proc movie::sync:withcheck {w time} {
variable $w
upvar 0 $w data

	set max [[routine::getRoutine $data(v:routineEditor)] get beats]
	if {$time > $max} {
		set time $max
	}
	sync:all $w $time
}

proc movie::sync:all {w time} {
variable $w
upvar 0 $w data

	# set the time for the view & update toolbar if we are at control point
	$data(w:view) set time $time
	set movie::${w}(v:time) $time
	if {$data(v:mode) == "view"} {
		if {[$data(w:view) info iscontrol $time]} {
			toolbar::configure $data(w:tb) delete -state normal
		} else {
			toolbar::configure $data(w:tb) delete -state disabled
		}
	}

	# sync the bodies in the view
	set routine [routine::getRoutine $data(v:routineEditor)]
	set i 0
	foreach b $data(v:bodies) {
		$routine applyto $i $time $b
		incr i
	}

	# refresh the notes
	sync:notes $w $time

	# refresh progress bar
	set beats [$routine get beats]
	if {$beats == 0} {
		set size 1
	} else {
		set size [expr ($time*$data(v:progress:length))/$beats]
	}
	$data(w:progress) coords $data(v:progress:id) 1 1 $size 14
	if {[expr ($time/1000)%2]} {
		$data(w:progress) itemconfigure $data(v:progress:id) -fill #0ea -outline #0ea
	} else {
		$data(w:progress) itemconfigure $data(v:progress:id) -fill #f99 -outline #f99
	}
}

proc movie::sync:notes {w {time -1}} {
variable $w
upvar 0 $w data

	if {$data(v:shownotes)} {
		set routine [routine::getRoutine $data(v:routineEditor)]
		set nb [$routine get dancers]
		if {$time == -1} {
			set time [$data(w:view) get time]
		}

		for {set i -1} {$i < $nb} {incr i} {
			$data(w:notes:$i) configure -text [$routine note text $time $i]
		}
	}
}

proc movie::sync:names {w} {
variable $w
upvar 0 $w data


	set routine [routine::getRoutine $data(v:routineEditor)]
	set nb [$routine get dancers]
	for {set i 0} {$i < $nb} {incr i} {
		$data(w:notes).$i configure -label [$routine get dancername $i]
	}
}

#-------------------------------------------------------------------------------------------------

proc movie::adjustTime {w deltaT} {
variable [winfo toplevel $w]
upvar 0 [winfo toplevel $w] data


	# adjust according to increment
	set max [[routine::getRoutine $data(v:routineEditor)] get beats]
	if {$deltaT == "zero"} {
		set time 0
	} elseif {$deltaT == "end"} {
		set time $max
	} else {
		set time [expr ([$data(w:view) get time]/50/$deltaT)*$deltaT*50]
		set time [expr $time + $deltaT*50]
	}
	if {$time < 0} {
		set time 0
	} elseif {$time > $max} {
		set time $max
	}

	sync:all [winfo toplevel $w] $time
}

#-------------------------------------------------------------------------------------------------

proc movie::set:shownotes {w on} {
variable $w
upvar 0 $w data

	set data(v:shownotes) $on

	# refresh/disable the notes
	if {[info exists data(v:routineEditor)]} {
		set routine [routine::getRoutine $data(v:routineEditor)]
		set nb [$routine get dancers]
		set time [$data(w:view) get time]
		for {set i -1} {$i < $nb} {incr i} {
			if {$on} {
				$data(w:notes:$i) configure -fg black -text [$routine note text $time $i]
			} else {
				$data(w:notes:$i) configure -fg #a3a3a3 -text "Disabled"
			}
		}
	}
}


#-------------------------------------------------------------------------------------------------

proc movie::record {w} {
variable $w
upvar 0 $w data

	$data(w:view) record [$data(w:view) get movie]
	$data(w:view) set time [$data(w:view) get time]		;# hack (if adding a point)
}

proc movie::delete {w} {
variable $w
upvar 0 $w data

	$data(w:view) modify [list eye [$data(w:view) get time] delete] 0 0 0
	toolbar::configure $data(w:tb) delete -state disabled
}

#-------------------------------------------------------------------------------------------------

proc movie::buildListMovies {w} {
variable $w
upvar 0 $w data


	$data(m:movies) delete 0 end
	set routine [routine::getRoutine $data(v:routineEditor)]
	set movies [lsort -dictionary [$routine get movies]]
	foreach m $movies {
		$data(m:movies) add command -label $m -command "movie::selectMovie $w [list $m]"
	}
	$data(m:movies) add separator
	$data(m:movies) add command -label "New movie ..." -command "movie::selectMovie $w {}"
	$data(m:movies) add command -label "Change name ..." -command "movie::changeMovie $w"
	$data(m:movies) add command -label "Delete current" -command "movie::deleteMovie $w"

	return [lindex $movies 0]
}

proc movie::deleteMovie {w} {
variable $w
upvar 0 $w data

	set routine [routine::getRoutine $data(v:routineEditor)]
	$data(w:view) set movie $routine {}
	$routine movie delete $data(v:movie)
	selectMovie $w [buildListMovies $w] 1
}

proc movie::selectMovie {w what {skip 0}} {
variable $w
upvar 0 $w data


	set data(v:movie) ""
	set routine [routine::getRoutine $data(v:routineEditor)]

	if {$what == {} && !$skip} {
		# get the name of the movie
		if {![getStringFromDialog "New movie" \
				"Choose the name to describe the movie" Name "" what]} {
			after idle "movie::set:shownotes $w 0"
			return
		}
		# create new movie
		$routine movie add $what
		# update list of available movies
		buildListMovies $w
		# reset time
		after idle "movie::sync:all $w 0"
	}

	# select movie to display
	set data(v:movie) $what
	if {$what != ""} {
		$data(w:view) set movie $routine $what
	}
}

proc movie::changeMovie {w} {
variable $w
upvar 0 $w data

	# if no movies defined return
	if {$data(v:movie) == {}} {
		return
	}
	# ask new name
	set routine [routine::getRoutine $data(v:routineEditor)]
	if {[getStringFromDialog "Movie" "Enter the new name for the movie" \
				Name $data(v:movie) name]} {
		if {[$routine movie changename $data(v:movie) $name]} {
			tk_messageBox -icon "error" -title "Error" \
					-message "Name already used." -type ok
			return
		}
		set data(v:movie) $name
		# update list with new name
		buildListMovies $w
	}
}

#-------------------------------------------------------------------------------------------------

proc movie::set:mode {w button} {
variable $w
upvar 0 $w data
variable help


	if {$data(v:mode) == "edit"} {
		set data(v:old3D) [$data(w:view) get all]
	}

	if {$data(mode) == "3d"} {
		set list [bindtags $data(w:view)]
		set i [lsearch $list "ViewMovie*"]
		if {$button == "view"} {
			bindtags $data(w:view) [lreplace $list $i $i "ViewMovieView"]
		} else {
			bindtags $data(w:view) [lreplace $list $i $i "ViewMovieEdit"]
		}
	}

	toolbar::configure $data(w:tb) delete -state disabled

	switch -exact $button {
		edit	{ toolbar::configure $data(w:tb) record -state disabled
				  $data(w:view) set mode edit
				  $data(w:view) set all $data(v:old3D)
				}
		view	{ toolbar::configure $data(w:tb) record -state normal
#				  toolbar::configure $data(w:tb) tobody -state normal
				  $data(w:view) set mode view
				  $data(w:view) viewpoint
				  if {[$data(w:view) info iscontrol [$data(w:view) get time]]} {
					  toolbar::configure $data(w:tb) delete -state normal
				  }
				}
		play	{ toolbar::configure $data(w:tb) record -state disabled
#				  toolbar::configure $data(w:tb) tobody -state disabled
				  $data(w:view) set mode play }
	}

	set help(normal) $help($data(v:mode):normal)
	set help(shift) $help($data(v:mode):shift)
	set help(control) $help($data(v:mode):control)
	set help(alt) $help($data(v:mode):alt)
}

#-------------------------------------------------------------------------------------------------

proc movie::motion {w x y} {
variable $w
upvar 0 $w data

	set what [$data(w:view) pick $x $y]
	if {$what != ""} {
		if {$data(v:mode) == "edit"} {
			if {$data(v:keystatus) == "normal"} {
				$data(w:view) configure -cursor fleur
			} else {
				$data(w:view) configure -cursor sb_v_double_arrow
			}
		} else {
			if {[lindex $what 2] == "point"} {
				$data(w:view) configure -cursor cross
			}
		}
	} else {
		$data(w:view) configure -cursor {}
	}
}

proc movie::modify:setup {w x y} {
variable $w
upvar 0 $w data


	set data(t:what) [$w pick $x $y]
	if {$data(t:what) == ""} {
		set data(t:ok) 0
	} else {
		set data(t:ok) 1
	    set data(t:az) [$w get azimut]
	    set data(t:zoom) [expr [$w get zoom]/5]
	    set data(t:x) $x
	    set data(t:y) $y
	}
	return $data(t:ok)
}

proc movie::modify {w x y type} {
variable $w
upvar 0 $w data

	if {$data(t:ok)} {
		if {$type == "xz"} {
		    set az [expr 3.141592654*$data(t:az)/180]
		    set tX [expr ($x-$data(t:x))*cos($az)-($y-$data(t:y))*sin($az)]
		    set tZ [expr ($x-$data(t:x))*sin($az)+($y-$data(t:y))*cos($az)]
		    set tX [expr $tX*0.01*exp(sqrt($data(t:zoom))/10)]
		    set tZ [expr $tZ*0.01*exp(sqrt($data(t:zoom))/10)]
			$w modify $data(t:what) $tX 0 $tZ
		} else {
			set tY [expr -$y+$data(t:y)]
			$w modify $data(t:what) 0 [expr $tY*0.01*exp(sqrt($data(t:zoom))/10)] 0
		}

	    set data(t:x) $x
	    set data(t:y) $y
	}
	return $data(t:ok)
}

proc movie::modify:proj:setup {w x y} {
variable [winfo toplevel $w]
upvar 0 [winfo toplevel $w] data


	set data(t:what) [$w pick $x $y]
	if {$data(t:what) == ""} {
		set data(t:ok) 0
	} else {
		set data(t:ok) 1
	    set data(t:zoom) [$w get zoom]
	    set data(t:x) $x
	    set data(t:y) $y
	}
	return $data(t:ok)
}

proc movie::modify:proj {w x y} {
variable [winfo toplevel $w]
upvar 0 [winfo toplevel $w] data

	if {$data(t:ok)} {
	    set tX [expr ($x-$data(t:x))*0.01*sqrt(1/$data(t:zoom))]
	    set tZ [expr ($y-$data(t:y))*0.01*sqrt(1/$data(t:zoom))]
		switch -exact $data(mode) {
			top		{ $w modify $data(t:what) $tX 0 $tZ }
			bottom	{ $w modify $data(t:what) $tX 0 [expr -$tZ] }
			front	{ $w modify $data(t:what) $tX [expr -$tZ] 0 }
			back	{ $w modify $data(t:what) [expr -$tX] [expr -$tZ] 0 }
			left	{ $w modify $data(t:what) 0 [expr -$tZ] $tX }
			right	{ $w modify $data(t:what) 0 [expr -$tZ] [expr -$tX] }
		}

	    set data(t:x) $x
	    set data(t:y) $y
	}
	return $data(t:ok)
}

#=================================================================================================

proc movie::actionMenu {w x y X Y} {
variable $w
upvar 0 $w data


	set what [$data(w:view) pick $x $y]
	if {[lindex $what 2] == "point"} {
		$w.mae entryconfigure 0 \
				-command "$data(w:view) modify [list [lreplace $what 2 2 align]] 0 0 0"
		$w.mae entryconfigure 1 \
				-command "$data(w:view) modify [list [lreplace $what 2 2 linearize]] 0 0 0"
		$w.mae entryconfigure 3 \
				-command "$data(w:view) modify [list [lreplace $what 2 2 delete]] 0 0 0"
		tk_popup $w.mae $X $Y
	} elseif {$data(v:mode) == "edit"} {
		tk_popup $w.mp $X $Y
	} elseif {$data(v:mode) == "view"} {
		tk_popup $w.mav $X $Y
	}
}

#-------------------------------------------------------------------------------------------------

proc movie::action:view {w what when} {
variable $w
upvar 0 $w data

	set time [$data(w:view) get time]

	# do center first as eye is computed relative to center
	if {$what == "center" || $what == "view"} {
#puts "info ${when}center $time = [$data(w:view) info ${when}center $time]"
		$data(w:view) set pos [$data(w:view) info ${when}center $time]
	}
	if {$what == "eye" || $what == "view"} {
#puts "info ${when}eye $time = [$data(w:view) info ${when}eye $time]"
		$data(w:view) set eye [$data(w:view) info ${when}eye $time]
	}
}


#=================================================================================================

# bindings for 3d view in EDIT mode
bind ViewMovieEdit <ButtonPress-1> \
	"if {!\[movie::modify:setup %W %x %y\]} {
		view3D::rotate:setup %W %x %y
	}"
bind ViewMovieEdit <Button1-Motion> \
	"if {!\[movie::modify %W %x %y xz\]} {
		view3D::rotate %W %x %y
	}"
bind ViewMovieEdit <Shift-ButtonPress-1> \
	"if {!\[movie::modify:setup %W %x %y\]} {
		view3D::zoom:setup %W %x %y
	}"
bind ViewMovieEdit <Shift-Button1-Motion> \
	"if {!\[movie::modify %W %x %y y\]} {
		view3D::zoom %W %x %y
	}"
bind ViewMovieEdit <Control-ButtonPress-1> \
	"if {!\[movie::modify:setup %W %x %y\]} {
		view3D::translate:setup %W %x %y
	}"
bind ViewMovieEdit <Control-Button1-Motion> \
	"if {!\[movie::modify %W %x %y y y\]} {
		view3D::translate %W %x %y
	}"

#bind ViewMovieEdit <ButtonPress-1> "view3D::rotate:setup %W %x %y"
#bind ViewMovieEdit <Button1-Motion> "view3D::rotate %W %x %y"
#bind ViewMovieEdit <Shift-ButtonPress-1> "view3D::zoom:setup %W %x %y"
#bind ViewMovieEdit <Shift-Button1-Motion> "view3D::zoom %W %x %y"
#bind ViewMovieEdit <Control-ButtonPress-1> "view3D::translate:setup %W %x %y"
#bind ViewMovieEdit <Control-Button1-Motion> "view3D::translate %W %x %y"
if {$tcl_platform(platform) == "windows"} {
	bind ViewMovieEdit <Mod2-ButtonPress-1> "view3D::center:setup %W %x %y x z"
	bind ViewMovieEdit <Mod2-Button1-Motion> "view3D::center %W %x %y x z"
} else {
	bind ViewMovieEdit <Mod1-ButtonPress-1> "view3D::center:setup %W %x %y x z"
	bind ViewMovieEdit <Mod1-Button1-Motion> "view3D::center %W %x %y x z"
}

# bindings for projection in EDIT mode
bind ViewMovieProjection <ButtonPress-1> \
	"if {!\[movie::modify:proj:setup %W %x %y\]} {
		view3D::translate:setup %W %x %y
	}"
bind ViewMovieProjection <Button1-Motion> \
	"if {!\[movie::modify:proj %W %x %y\]} {
		view3D::projTranslate %W %x %y
	}"

#bind ViewMovieProjection <ButtonPress-1> "view3D::translate:setup %W %x %y"
#bind ViewMovieProjection <Button1-Motion> "view3D::projTranslate %W %x %y"
bind ViewMovieProjection <Shift-ButtonPress-1> "view3D::zoom:setup %W %x %y"
bind ViewMovieProjection <Shift-Button1-Motion> "view3D::zoom %W %x %y"
#bind ViewMovieProjection <ButtonPress-2> "movie::modify:proj:setup %W %x %y"
#bind ViewMovieProjection <Button2-Motion> "movie::modify:proj %W %x %y"

#-----------------------
# bindings for VIEW mode
bind ViewMovieView <ButtonPress-1> "movie::rotate:setup %W %x %y"
bind ViewMovieView <Button1-Motion> "movie::rotate %W %x %y"
bind ViewMovieView <Control-ButtonPress-1> "view3D::rotate:setup %W %x %y"
bind ViewMovieView <Control-Button1-Motion> "view3D::rotate %W %x %y"

bind ViewMovieView <Shift-ButtonPress-1> "view3D::zoom:setup %W %x %y"
bind ViewMovieView <Shift-Button1-Motion> "view3D::zoom %W %x %y"
if {$tcl_platform(platform) == "windows"} {
	bind ViewMovieView <Mod2-ButtonPress-1> "view3D::center:setup %W %x %y x z"
	bind ViewMovieView <Mod2-Button1-Motion> "view3D::center %W %x %y x z"
	bind ViewMovieView <Shift-Mod2-ButtonPress-1> "view3D::center:setup %W %x %y y y"
	bind ViewMovieView <Shift-Mod2-Button1-Motion> "view3D::center %W %x %y y y"
} else {
	bind ViewMovieView <Mod1-ButtonPress-1> "view3D::center:setup %W %x %y x z"
	bind ViewMovieView <Mod1-Button1-Motion> "view3D::center %W %x %y x z"
	bind ViewMovieView <Shift-Mod1-ButtonPress-1> "view3D::center:setup %W %x %y y y"
	bind ViewMovieView <Shift-Mod1-Button1-Motion> "view3D::center %W %x %y y y"
}

# bind ViewMovieView <ButtonPress-2> "view3D::rotate:setup %W %x %y"
# bind ViewMovieView <Button2-Motion> "view3D::rotate %W %x %y"
# bind ViewMovieView <Control-ButtonPress-2> "view3D::center:setup %W %x %y y y"
# bind ViewMovieView <Control-Button2-Motion> "view3D::center %W %x %y y y"
# if {$tcl_platform(platform) == "windows"} {
# 	bind ViewMovieView <Mod2-ButtonPress-2> "view3D::center:setup %W %x %y y y"
# 	bind ViewMovieView <Mod2-Button2-Motion> "view3D::center %W %x %y y y"
# } else {
# 	bind ViewMovieView <Mod1-ButtonPress-2> "view3D::center:setup %W %x %y y y"
# 	bind ViewMovieView <Mod1-Button2-Motion> "view3D::center %W %x %y y y"
# }


#-----------------------
# bindings for the time
bind KeyMovie <Left> "movie::adjustTime %W -5"
bind KeyMovie <Right> "movie::adjustTime %W +5"
bind KeyMovie <Shift-Left> "movie::adjustTime %W -1"
bind KeyMovie <Shift-Right> "movie::adjustTime %W +1"
if {$tcl_platform(platform) == "windows"} {
	bind KeyMovie <Mod2-Left> "movie::adjustTime %W -20"
	bind KeyMovie <Mod2-Right> "movie::adjustTime %W +20"
} else {
	bind KeyMovie <Mod1-Left> "movie::adjustTime %W -20"
	bind KeyMovie <Mod1-Right> "movie::adjustTime %W +20"
}
bind KeyMovie <Up> "movie::adjustTime %W end"
bind KeyMovie <Down> "movie::adjustTime %W zero"


#-------------------------------------------------------------------------------------------------

proc movie::initView {w} {
variable $w
upvar 0 $w data

	set data(mode:old) 3d
	set data(mode) 3d

	#					   az  el  tw  zoom  xt  yt
	set data(var:3d)	 {-35  22   0	 7	  0   0}
	set data(var:front)  {  0   0   0	.16	  0  -1}
    set data(var:back)   {180   0   0	.16   0  -1}
    set data(var:left)   { 90   0   0	.16   0  -1}
    set data(var:right)  {-90   0   0	.16   0  -1}
    set data(var:top)    {  0  90   0	.16   0   0}
    set data(var:bottom) {  0 -90   0	.16   0   0}

	$data(w:view) set view $data(var:3d)


    menu $w.mp -tearoff 0 -font {Hevetica 12} -bd 1 -activeborderwidth 1
      $w.mp add radio -label "3d" -command "movie::view $w 3d" \
	      -variable movie::${w}(mode) -value 3d
      $w.mp add radio -label "Top" -command "movie::view $w top" \
	      -variable movie::${w}(mode) -value top
      $w.mp add radio -label "Bottom" -command "movie::view $w bottom" \
	      -variable movie::${w}(mode) -value bottom
      $w.mp add radio -label "Front" -command "movie::view $w front" \
	      -variable movie::${w}(mode) -value front
      $w.mp add radio -label "Back" -command "movie::view $w back" \
	      -variable movie::${w}(mode) -value back
      $w.mp add radio -label "Left" -command "movie::view $w left" \
	      -variable movie::${w}(mode) -value left
      $w.mp add radio -label "Right" -command "movie::view $w right" \
	      -variable movie::${w}(mode) -value right
      $w.mp add separator
	  set ${w}(floor) 1
	  set ${w}(axis) 1
	  $w.mp add check -label "Floor" -variable movie::${w}(floor) \
		      -command "$data(w:view) set floor \[set movie::${w}(floor)\]"
	  $w.mp add check -label "Axis" -variable movie::${w}(axis) \
		      -command "$data(w:view) set axis \[set movie::${w}(axis)\]"
}

#-------------------------------------------------------------------------------------------------

proc movie::view {w mode} {
variable $w
upvar 0 $w data
variable help

	set old $data(mode:old)
	set data(mode:old) $mode

	set data(var:$old) [$data(w:view) get view]

	if {$mode == "3d"} {
		# 3d
		$data(w:view) persp

		set list [bindtags $data(w:view)]
		set i [lsearch -exact $list "ViewMovieProjection"]
		if {$data(v:mode) == "view"} {
			bindtags $data(w:view) [lreplace $list $i $i "ViewMovieView"]
		} else {
			bindtags $data(w:view) [lreplace $list $i $i "ViewMovieEdit"]
		}

		set help(normal) $help($data(v:mode):normal)
		set help(shift) $help($data(v:mode):shift)
		set help(control) $help($data(v:mode):control)
		set help(alt) $help($data(v:mode):alt)
	} else {
		# projection
		$data(w:view) ortho

		set list [bindtags $data(w:view)]
		set i [lsearch $list "ViewMovie*"]
		bindtags $data(w:view) [lreplace $list $i $i "ViewMovieProjection"]

		set help(normal) $help(proj:normal)
		set help(shift) $help(proj:shift)
		set help(control) $help(proj:control)
		set help(alt) $help(proj:alt)
	}

	$data(w:view) set view $data(var:$mode)
}

proc movie::view:reset {w action} {
variable $w
upvar 0 $w data


	switch -exact $action {
		tobody			{ $data(w:view) set xtrans [lindex $data(var:$data(mode)) 4]
						  $data(w:view) set ytrans [lindex $data(var:$data(mode)) 5] 
						  set xpos 0
						  set zpos 0
						  foreach b $data(v:bodies) {
							  set xpos [expr $xpos + [$b get cog:X]]
							  set zpos [expr $zpos + [$b get cog:Z]]
						  }
						  $data(w:view) set xpos [expr $xpos/100/[llength $data(v:bodies)]]
						  $data(w:view) set zpos [expr $zpos/100/[llength $data(v:bodies)]]
						  $data(w:view) set ypos 0
						}
	}
}

#-------------------------------------------------------------------------------------------------

### Rotate from the eye position
proc movie::rotate:setup {w x y} {
variable $w
upvar 0 $w data

    set data(t:X) $x
    set data(t:Y) $y
    set data(t:Az) [$w get azimut]
    set data(t:El) [$w get elevation]
	set data(t:dist) [$w get zoom]
    set data(t:Xcenter) [$w get xpos]
    set data(t:Ycenter) [$w get ypos]
    set data(t:Zcenter) [$w get zpos]

	set az [expr $data(t:Az)*3.141592654/180.0]
	set el [expr $data(t:El)*3.141592654/180.0]
	set data(t:Xeye) [expr - sin($az)*cos($el)*$data(t:dist)]
	set data(t:Yeye) [expr +          sin($el)*$data(t:dist)]
	set data(t:Zeye) [expr + cos($az)*cos($el)*$data(t:dist)]
}

proc movie::rotate {w x y} {
variable $w
upvar 0 $w data

    set az [expr ($x-$data(t:X))/2.0+$data(t:Az)]
    if {$az<-360} then {set az [expr $az+360]}
    if {$az>360} then {set az [expr $az-360]}
    set el [expr ($y-$data(t:Y))/4.0+$data(t:El)]
    if {$el>90} then {set el 90}
    if {$el<-90} then {set el -90}
    $w set azimut $az
    $w set elevation $el

	set az [expr $az*3.141592654/180.0]
	set el [expr $el*3.141592654/180.0]

	set dX [expr - sin($az)*cos($el)*$data(t:dist) - $data(t:Xeye)]
	set dY [expr +          sin($el)*$data(t:dist) - $data(t:Yeye)]
	set dZ [expr + cos($az)*cos($el)*$data(t:dist) - $data(t:Zeye)]
#puts "$dX  $dY  $dZ"
	$w set xpos [expr $data(t:Xcenter) - $dX]
	$w set ypos [expr $data(t:Ycenter) - $dY]
	$w set zpos [expr $data(t:Zcenter) - $dZ]

#puts "---------------"
#puts "eye = <[expr [$w get xpos]+sin($az)*cos($el)*$data(t:dist)],[expr [$w get ypos]+sin($el)*$data(t:dist)],[expr [$w get zpos]-cos($az)*cos($el)*$data(t:dist)]>"
#puts "center = <[$w get xpos],[$w get ypos],[$w get zpos]>"
}


#=================================================================================================

# array set movie::help {
# 	normal			"<1> Rotate view    <2> Move control in plane XZ    <3> Action menu"
# 	shift			"<1> Zoom view    <2> Move control along Y    <3> Action menu"
# 	control			"<1> Translate view    <2> Move control along Y    <3> Action menu"
# 	alt				"<1> Translate center    <2> Move control in plane XZ    <3> Action menu"

# 	proj:normal		"<1> Translate view    <2> Move control in plane     <3> Action menu"
# 	proj:shift		"<1> Zoom view    <2> Move control in plane    <3> Action menu"
# 	proj:control	"<1> Translate view    <2> Move control in plane    <3> Action menu"
# 	proj:alt		"<1> Translate view    <2> Move control in plane    <3> Action menu"

# 	edit:normal		"<1> Rotate view    <2> Move control in plane XZ    <3> Action menu"
# 	edit:shift		"<1> Zoom view    <2> Move control along Y    <3> Action menu"
# 	edit:control	"<1> Translate view    <2> Move control along Y    <3> Action menu"
# 	edit:alt		"<1> Translate center    <2> Move control in plane XZ    <3> Action menu"

# 	view:normal		"<1> Rotate from eye    <2> Rotate from center"
# 	view:shift		"<1> Zoom view    <2> Rotate from center"
# 	view:control	"<1> Translate XZ    <2> Translate Y"
# 	view:alt		"<1> Translate Y    <2> Translate Y"

# 	play:normal		""
# 	play:shift		""
# 	play:control	""
# 	play:alt		""

# 	off		""}
array set movie::help {
	normal			"<L> Rotate view    <L-obj> Move control in plane XZ    <R> Action menu"
	shift			"<L> Zoom view    <L-obj> Move control along Y    <R> Action menu"
	control			"<L> Translate view    <L-obj> Move control along Y    <R> Action menu"
	alt				"<L> Translate center    <L-obj> Move control in plane XZ    <R> Action menu"

	proj:normal		"<L> Translate view    <L-obj> Move control in plane     <R> Action menu"
	proj:shift		"<L> Zoom view    <L-obj> Move control in plane    <R> Action menu"
	proj:control	"<L> Translate view    <L-obj> Move control in plane    <R> Action menu"
	proj:alt		"<L> Translate view    <L-obj> Move control in plane    <R> Action menu"

	edit:normal		"<L> Rotate view    <L-obj> Move control in plane XZ    <R> Action menu"
	edit:shift		"<L> Zoom view    <L-obj> Move control along Y    <R> Action menu"
	edit:control	"<L> Translate view    <L-obj> Move control along Y    <R> Action menu"
	edit:alt		"<L> Translate center    <L-obj> Move control in plane XZ    <R> Action menu"

	view:normal		"<L> Rotate from eye    <R> Action menu"
	view:shift		"<L> Zoom view    <R> Action menu"
	view:control	"<L> Rotate from center    <R> Action menu"
	view:alt		"<L> Translate XZ (press Shift with Alt to Translate Y)   <R> Action menu"

	play:normal		""
	play:shift		""
	play:control	""
	play:alt		""

	off		""}



################################      D U M M Y       ############################################


### Translate the eye inside the plane defined by Axe1/Axe2 
proc movie::eye:setup {w x y} {
variable $w
upvar 0 $w data

    set data(t:x) $x
    set data(t:y) $y
    set data(t:az) [$w get azimut]
    set data(t:zoom) [expr [$w get zoom]/5]
    set data(t:eye) [$w get eye]
}

proc movie::eye {w x y axe} {
variable $w
upvar 0 $w data

	if {$axe == "xz"} {
	    set az [expr 3.141592654*$data(t:az)/180]
	    set tX [expr -($y-$data(t:y))*sin($az)-($x-$data(t:x))*cos($az)]
	    set tZ [expr ($y-$data(t:y))*cos($az)-($x-$data(t:x))*sin($az)]
	    set tX [expr $tX * 0.01*exp(sqrt($data(t:zoom))/10) + [lindex $data(t:eye) 0]]
	    set tZ [expr $tZ * 0.01*exp(sqrt($data(t:zoom))/10) + [lindex $data(t:eye) 2]]
	    set list [lreplace $data(t:eye) 0 0 $tX]
	    set list [lreplace $data(t:eye) 2 2 $tZ]
	    $w set eye $list 
	} else {
		set tY [expr -($y-$data(t:y))* 0.01*exp(sqrt($data(t:zoom))/10) + [lindex $data(t:eye) 1]]
	    set list [lreplace $data(t:eye) 1 1 $tY]
	    $w set eye $list 
	}
}
