%option yylineno

%{
extern "C" int fileno( FILE *stream); //#include <stdio.h>

#include "togl.h"
#include "global.h"

#include "parser.tab.cc.h"

extern char *trackName[];
extern char *jointName[];


#define MAX_STR_CONST 10*1024
static char string_buf[MAX_STR_CONST];
static char *string_buf_ptr;

#ifdef WIN32
#include <malloc.h>
#define alloca _alloca
#endif
%}

comment		#.*
blank		[ \t\r\f\n]*
digit		[0-9]
sign		[-+]
joint		pelvis|torso|head|(left|right):(shoulder|elbow|wrist|hip|knee|ankle|foot)|cog:(position|altitude|roll|orientation)
track		pelvis:[XYZ]|torso:[XYZ]|head:[XYZ]|(left|right):(shoulder|elbow|wrist|hip|knee|ankle|foot):(TY|[XYZ])|cog:(r)?[XYZ]
filename	[A-Za-z0-9/_.]+
 
%x STATE_STRING
     
%%

"dancers"		return tDANCERS;
"beats"			return tBEATS;
"rhythm"		return tRHYTHM;
"bpm"			return tBPM;
"name"			return tNAME;
"description"	return tDESCRIPTION;
"copyright"		return tCOPYRIGHT;
"text"			return tTEXT;

"man"			return tMAN;
"woman"			return tWOMAN;

"init"			return tINIT;

"key"			return tKEY;
"linear"		return tLINEAR;
"bezier"		return tBEZIER;
"handles"		return tHANDLES;
"handleleft"	return tHANDLELEFT;
"handleright"	return tHANDLERIGHT;

"routine"		return tROUTINE;
"map"			return tMAP;
"with"			return tWITH;
"reject"		return tREJECT;
"color"			return tCOLOR;
"includenotes"	return tINCLUDENOTES;

"notes"			return tNOTES;

"movie"			return tMOVIE;
"eye"			return tEYE;
"center"		return tCENTER;


{joint} {
		for (yylval.integer=0; yylval.integer<NB_JOINTS; yylval.integer++) {
			if (strcmp(yytext, jointName[yylval.integer]) == 0)
				return tJOINTNAME;
		}
	}

{track} {
		for (yylval.integer=0; yylval.integer<NB_DOF; yylval.integer++) {
			if (strcmp(yytext, trackName[yylval.integer]) == 0)
				return tTRACKNAME;
		}
	}

{sign}?{digit}+ {
		sscanf(yytext, "%d", &yylval.integer);
		return tINTEGER;
	}
 
{sign}?{digit}*"."{digit}+[eE]?{sign}?{digit}* {
		sscanf(yytext, "%lf", &yylval.numeric);
		return tDOUBLE;
	}

{filename} {
		yylval.string = new char[strlen(yytext)+1];
		strcpy(yylval.string, yytext);
		return tFILENAME;
	}

{comment}			/* eat up one-line comments */

{blank}				/* eat up whitespace */

\"	{
		string_buf_ptr = string_buf;
		BEGIN(STATE_STRING);
	}

. 		return yytext[0];

<STATE_STRING>{
	\" {
		/* saw closing quote - all done */
		BEGIN(INITIAL);
		*string_buf_ptr = '\0';
		/* return string constant token type and
		 * value to parser
		 */
		yylval.string = new char[strlen(string_buf)+1];
		strcpy(yylval.string, string_buf);
		return tSTRING;
	}

	\n {
		/* error - unterminated string constant */
		/* generate error message */
	}

	\\[0-7]{1,3} {
		/* octal escape sequence */
		int value = yytext[1] - '0';
		char *cursor = yytext + 2;
    
    	while (*cursor)
			value = 8 * value + *cursor++ - '0';

		*string_buf_ptr++ = value;
	}

	\\x[0-9a-fA-F]{1,2} {
	    int value = 0;
	    char *cursor = yytext + 2;
    
	    while (*cursor) {
			if (*cursor >= 'a' && *cursor <= 'f')
				value = 16 * value + *cursor++ - 'a' + 10;
			else if (*cursor >= 'A' && *cursor <= 'F')
				value = 16 * value + *cursor++ - 'A' + 10;
			else
				value = 16 * value + *cursor++ - '0';
		}

		*string_buf_ptr++ = value;
	}

	\\[0-9]+ {
		/* generate error - bad escape sequence; something
		 * like '\48' or '\0777777'
		 */
	}

	\\n			{ *string_buf_ptr++ = '\n'; }
	\\t			{ *string_buf_ptr++ = '\t'; }
	\\r			{ *string_buf_ptr++ = '\r'; }
	\\b			{ *string_buf_ptr++ = '\b'; }
	\\f			{ *string_buf_ptr++ = '\f'; }

	\\(.|\n)	{ *string_buf_ptr++ = yytext[1]; }

	[^\\\n\"]+ {
		char *yptr = yytext;
		while (*yptr)
			*string_buf_ptr++ = *yptr++;
	}
}

%%

int yywrap(void)
{
	return 1;
}
