namespace eval player {
	variable list {}

	image create pixmap mm_reverse -file pixmap/mm_reverse.xpm
	image create pixmap mm_reverse_pushed -file pixmap/mm_reverse_pushed.xpm
	image create pixmap mm_play -file pixmap/mm_play.xpm
	image create pixmap mm_play_pushed -file pixmap/mm_play_pushed.xpm
	image create pixmap mm_stop -file pixmap/mm_stop.xpm
	image create pixmap mm_stop_pushed -file pixmap/mm_stop_pushed.xpm
	image create pixmap mm_pause -file pixmap/mm_pause.xpm
	image create pixmap mm_pause_pushed -file pixmap/mm_pause_pushed.xpm
	image create pixmap mm_start -file pixmap/mm_start.xpm
	image create pixmap mm_end -file pixmap/mm_end.xpm
	image create pixmap mm_backward -file pixmap/mm_backward.xpm
	image create pixmap mm_forward -file pixmap/mm_forward.xpm
}

#=================================================================================================

proc player::new {w callback {gettime ""}} {
variable $w
upvar 0 $w data


	# create the row of buttons
	foreach b {mm_start mm_backward mm_reverse mm_pause mm_stop \
			   mm_play mm_forward mm_end} {

		button $w.$b -image $b -bd 0 -activebackground lightgray \
				-relief flat -highlightthickness 0 \
				-command "player::invoke $w $b"
		if {$b == "mm_backward" || $b == "mm_forward"} {
			$w.$b configure -command ""
			bind $w.$b <ButtonPress-1> "player::invoke $w $b"
			bind $w.$b <ButtonRelease-1> "after cancel \[set player::${w}(t:repeat)\]"
		}
		pack $w.$b -side left -padx 0
	}

	$w.mm_stop configure -image mm_stop_pushed
	set data(v:mode) "stop"

	# some init
	set data(t:pending) -1
	set data(t:repeat) -1
	set data(t:timeincr) 0
	set data(t:currentbeat) 0

	set data(v:sync) $callback
	set data(v:routine) ""
	set data(v:repeatSpeed) 200
	set data(v:repeatIncr) 1000

	if {$gettime == ""} {
		set data(v:gettime) "$data(v:routine)"
	} else {
		set data(v:gettime) $gettime
	}
}

#-------------------------------------------------------------------------------------------------

proc player::invoke {w cmd} {
variable $w
upvar 0 $w data


	if {$data(v:routine) == ""} {
		return
	}


	switch -exact $cmd {
		mm_stop		  {	$w.mm_play configure -image mm_play
						$w.mm_reverse configure -image mm_reverse
						$w.mm_pause configure -image mm_pause
						$w.mm_stop configure -image mm_stop_pushed

						set data(v:mode) "stop"
					  	after cancel $data(t:pending)
						set data(t:pending) -1
						set data(t:timeincr) 0
					  }

		mm_play		  {	if {$data(v:mode) == "play"} {
							return
						}
						$w.mm_stop configure -image mm_stop
						$w.mm_reverse configure -image mm_reverse
						$w.mm_play configure -image mm_play_pushed

						if {[$w.mm_pause cget -image] == "mm_pause_pushed"} {
							invoke $w mm_pause
						}
						set data(v:mode) "play"
						computeTiming $w
					  	catch {after cancel $data(t:pending)}
						set data(t:pending) [after $data(t:speed) "player::sync $w"]
					  }

		mm_reverse	  {	if {$data(v:mode) == "reverse"} {
							return
						}
						$w.mm_stop configure -image mm_stop
						$w.mm_reverse configure -image mm_reverse_pushed
						$w.mm_play configure -image mm_play

						if {[$w.mm_pause cget -image] == "mm_pause_pushed"} {
							invoke $w mm_pause
						}
						set data(v:mode) "reverse"
						computeTiming $w
						set data(t:timeincr) [expr -$data(t:timeincr)]
					  	catch {after cancel $data(t:pending)}
						set data(t:pending) [after $data(t:speed) "player::sync $w"]
					  }

		mm_pause	  {	if {[$w.mm_pause cget -image] == "mm_pause_pushed"} {
							$w.mm_pause configure -image mm_pause
							set data(t:pending) [player::sync $w]
						} elseif {$data(v:mode) != "stop"} {
							$w.mm_pause configure -image mm_pause_pushed
						  	after cancel $data(t:pending)
							set data(t:pending) -1
						}
					  }

		mm_backward	  { set data(t:repeat) [after $data(v:repeatSpeed) \
												"player::invoke $w mm_backward"]
						incr data(t:time) -$data(v:repeatIncr)
						sync $w 0
					  }
		mm_forward	  { set data(t:repeat) [after $data(v:repeatSpeed) \
												"player::invoke $w mm_forward"]
						incr data(t:time) $data(v:repeatIncr)
						sync $w 0
					  }

		mm_start	  { set data(t:time) 0
						sync $w 0 }
		mm_end		  { set data(t:time) [$data(v:routine) get beats]
						sync $w 0 }
	}
}

#-------------------------------------------------------------------------------------------------

proc player::computeTiming {w} {
variable $w
upvar 0 $w data

	set data(t:time) [$data(v:gettime) get time]
	scan [time "eval $data(v:sync) $data(t:time); \
			update idletasks"] "%d" data(t:speed)
#puts -nonewline "time = $data(t:speed)    bpm = [$data(v:routine) get bpm],[$data(v:routine) get rhythm]    "

	set data(t:timeincr) [expr int( \
			[$data(v:routine) get bpm]*[$data(v:routine) get rhythm]/   \
			60000*$data(t:speed)*0.00125)]
	set data(t:speed) [expr int($data(t:speed)*0.00125)]
#puts "speed = $data(t:speed)      incr = $data(t:timeincr)"
}

#=================================================================================================

proc player::sync {w {next 1}} {
variable $w
upvar 0 $w data


	incr data(t:time) $data(t:timeincr)
	if {$data(t:time) >= [$data(v:routine) get beats]} {
		set data(t:time) [$data(v:routine) get beats]
		after idle "$w.mm_stop invoke"
	} elseif {$data(t:time) <= 0} {
		set data(t:time) 0
		after idle "$w.mm_stop invoke"
	} elseif {$next} {
		set data(t:pending) [after $data(t:speed) "player::sync $w"]
	}

	# beep for each beat
	#set beat [expr $data(t:time)/1000]
	#if {$data(t:currentbeat) != $beat} {
	#	bell
	#	set data(t:currentbeat) $beat
	#}

	eval $data(v:sync) $data(t:time)
}

#-------------------------------------------------------------------------------------------------

proc player::setRoutine {w routine} {
variable $w
upvar 0 $w data

	set data(v:routine) $routine
	if {$data(v:gettime) == ""} {
		set data(v:gettime) $routine
	}
	set data(t:time) [$data(v:gettime) get time]
}

#-------------------------------------------------------------------------------------------------

proc player::toggle {w mode} {
variable $w
upvar 0 $w data


	if {$data(v:mode) == "stop"} {
		invoke $w mm_$mode
	} else {
		invoke $w mm_stop
	}
}
