namespace eval view3D {
}


proc view3D::flip {w az el} {
    if {$az==0 && $el==0} {
		# Plane XY
		if {[$w get azimut] == 0} {
		    $w set azimut 180
		} else {
		    $w set azimut 0
		}
    } elseif {$az==90 && $el==0} {
		# Plane YZ
		$w set azimut [expr -[$w get azimut]]
    } else {
		# Plane XZ
		$w set elevation [expr -[$w get elevation]]
    }
}


#=================================================================================================
#
#       Setup bindings
#
#=================================================================================================

bind View3DProjection <ButtonPress-1> "view3D::translate:setup %W %x %y"
bind View3DProjection <Button1-Motion> "view3D::projTranslate %W %x %y"
bind View3DProjection <Shift-ButtonPress-1> "view3D::zoom:setup %W %x %y"
bind View3DProjection <Shift-Button1-Motion> "view3D::zoom %W %x %y"


bind View3D <ButtonPress-1> "view3D::rotate:setup %W %x %y"
bind View3D <Button1-Motion> "view3D::rotate %W %x %y"
bind View3D <Shift-ButtonPress-1> "view3D::zoom:setup %W %x %y"
bind View3D <Shift-Button1-Motion> "view3D::zoom %W %x %y"
bind View3D <Control-ButtonPress-1> "view3D::translate:setup %W %x %y"
bind View3D <Control-Button1-Motion> "view3D::translate %W %x %y"
if {$tcl_platform(platform) == "windows"} {
	bind View3D <Mod2-ButtonPress-1> "view3D::center:setup %W %x %y x z"
	bind View3D <Mod2-Button1-Motion> "view3D::center %W %x %y x z"
} else {
	bind View3D <Mod1-ButtonPress-1> "view3D::center:setup %W %x %y x z"
	bind View3D <Mod1-Button1-Motion> "view3D::center %W %x %y x z"
}




### Rotate around object
proc view3D::rotate:setup {w x y} {
variable $w
upvar 0 $w data

    set data(t:X) $x
    set data(t:Y) $y
    set data(t:Az) [$w get azimut]
    set data(t:El) [$w get elevation]
}

proc view3D::rotate {w x y} {
variable $w
upvar 0 $w data

    set az [expr ($x-$data(t:X))/2.0+$data(t:Az)]
    if {$az<-360} then {set az [expr $az+360]}
    if {$az>360} then {set az [expr $az-360]}
    set el [expr ($y-$data(t:Y))/4.0+$data(t:El)]
    if {$el>90} then {set el 90}
    if {$el<-90} then {set el -90}
    $w set azimut $az
    $w set elevation $el
}

### Translate the screen
proc view3D::translate:setup {w x y} {
variable $w
upvar 0 $w data

    set data(t:X) $x
    set data(t:Y) $y
    set data(t:tX) [$w get xtrans]
    set data(t:tY) [$w get ytrans]
    set data(t:zoom) [$w get zoom]
}

proc view3D::translate {w x y} {
variable $w
upvar 0 $w data

    set tX [expr ($x-$data(t:X))*0.01*exp(sqrt($data(t:zoom))/10.0) +$data(t:tX)]
    set tY [expr (-$y+$data(t:Y))*0.01*exp(sqrt($data(t:zoom))/10.0) +$data(t:tY)]
    $w set xtrans $tX
    $w set ytrans $tY
}

proc view3D::projTranslate {w x y} {
variable $w
upvar 0 $w data

    set tX [expr ($x-$data(t:X))*0.01*sqrt(1/$data(t:zoom)) +$data(t:tX)]
    set tY [expr (-$y+$data(t:Y))*0.01*sqrt(1/$data(t:zoom)) +$data(t:tY)]
    $w set xtrans $tX
    $w set ytrans $tY
}

### Translate the center inside the plane defined by Axe1/Axe2 
proc view3D::center:setup {w x y axe1 axe2} {
variable $w
upvar 0 $w data

    set data(t:X) $x
    set data(t:Y) $y
    set data(t:az) [$w get azimut]
    set data(t:zoom) [expr [$w get zoom]/5.0]
    set data(t:axe1) [$w get [set axe1]pos]
    set data(t:axe2) [$w get [set axe2]pos]
}

proc view3D::center {w x y axe1 axe2} {
variable $w
upvar 0 $w data

	if {$axe1 == $axe2} {
	    set tX [expr $y-$data(t:Y)]
	    set tX [expr -$tX * 0.01*exp(sqrt($data(t:zoom))/10.0) +$data(t:axe1)]		
	    $w set [set axe1]pos $tX
	} else {
	    set az [expr 3.141592654*$data(t:az)/180]
	    set tX [expr -($y-$data(t:Y))*sin($az)-($x-$data(t:X))*cos($az)]
	    set tZ [expr  ($y-$data(t:Y))*cos($az)-($x-$data(t:X))*sin($az)]
	    set tX [expr -$tX * 0.01*exp(sqrt($data(t:zoom))/10.0) +$data(t:axe1)]
	    set tZ [expr -$tZ * 0.01*exp(sqrt($data(t:zoom))/10.0) +$data(t:axe2)]
	    $w set [set axe1]pos $tX
	    $w set [set axe2]pos $tZ
	}
}

### Zoom
proc view3D::zoom:setup {w x y} {
variable $w
upvar 0 $w data

    set data(t:Y) $y
    set data(t:zY) [$w get zoom]
}

proc view3D::zoom {w x y} {
variable $w
upvar 0 $w data

    set zY [expr exp(($y-$data(t:Y))*.025)*$data(t:zY)]
    $w set zoom $zY
}


### Picking an object in a projection window
proc view3D::pickProj:setup {wdata w x y axe} {
variable $wdata
upvar 0 $wdata data

    set what [$w pickjoint $x $y]
#puts "pick proj : $x,$y --> $what"
    if {$what != ""} {
		set data(body) [lindex $what 0]
		set data(joint) [lindex $what 1]
		set data(x) $x
		set data(y) $y
		if {[catch {set data(j) [$data(body) get $data(joint):$axe]} msg]} {
#puts "*** error : $data(joint):$axe"
		    set data(t:ok) 2
		} else {
		    set data(t:ok) 1
		}
	} else {
		set data(t:ok) 0
    }
	return $data(t:ok)
}

proc view3D::pickProj {wdata w x y axe} {
variable $wdata
upvar 0 $wdata data

    if {$data(t:ok) == 1} {
		set theta [$data(body) get cog:rY]
		if {($theta > 90 || $theta < -90) && ![$w get jointsonly]} {
		    set inverse -1
		} else {
		    set inverse 1
		}
		if {[expr abs($x-$data(x))] > [expr abs($y-$data(y))]} {
			set j [expr ($x-$data(x)+$data(j)) * $data(dir:$axe)*$inverse]
		} else {
			set j [expr ($y-$data(y)+$data(j)) * $data(dir:$axe)*$inverse]
		}
		$data(body) cset $data(joint):$axe $j
    }
	return $data(t:ok)
}


### Picking an object in a 3d window
proc view3D::pick3D:setup {wdata w x y type {mode {}} {reset yes}} {
variable $wdata
upvar 0 $wdata data

    set what [$w pickjoint $x $y]
    if {$what != ""} {
		# setup internal/temp variables
		set data(t:body) [lindex $what 0]
		set data(t:joint) [lindex $what 1]
		set data(t:x) $x
		set data(t:y) $y
	    set data(t:zoom) [expr [$w get zoom]/5.0]
		set data(t:az) [$w get azimut]
		set data(t:mode) $mode
	    set data(t:axe) ""
	    set data(t:ok) 1
		if {[catch {set data(t:jX) [$data(t:body) get $data(t:joint):X]}]} {set data(t:jX) 0}
		if {[catch {set data(t:jY) [$data(t:body) get $data(t:joint):Y]}]} {set data(t:jY) 0}
		if {[catch {set data(t:jZ) [$data(t:body) get $data(t:joint):Z]}]} {set data(t:jZ) 0}
		# catch some special cases
		if {$reset == "yes" && [regexp {.*:(shoulder|hip|wrist|ankle)} $data(t:joint)]} {
			$data(t:body) set $data(t:joint):Y 0
		}
		# display guide (sphere/disk)
		$w set size .25
		$w set $type$mode true

	} else {
		set data(t:ok) 0
    }
	return $data(t:ok)
}

proc view3D::pick3D:xz {wdata x y type {mode {}}} {
variable $wdata
upvar 0 $wdata data

	if {$data(t:ok)} {
	    set az [expr 3.141592654*$data(t:az)/180]
	    set jX [expr -(-$y+$data(t:y))*cos($az)-(-$x+$data(t:x))*sin($az)]
	    set jZ [expr  (-$y+$data(t:y))*sin($az)-(-$x+$data(t:x))*cos($az)]
	    set jX [expr $jX * exp(sqrt($data(t:zoom))/10.0) + $data(t:jX)]
	    set jZ [expr $jZ * exp(sqrt($data(t:zoom))/10.0) + $data(t:jZ)]
#puts "jX = $jX -- jZ = $jZ"

		if {$data(t:mode) != "Z"} { catch { $data(t:body) cset $data(t:joint):X $jX } }
		if {$data(t:mode) != "X"} { catch { $data(t:body) cset $data(t:joint):Z $jZ } }
	}
	return $data(t:ok)
}

proc view3D::pick3D:y {wdata x y type} {
variable $wdata
upvar 0 $wdata data

	if {$data(t:ok)} {
#	    set az [expr 3.141592654*$data(t:az)/180]
#	    set jX [expr ($y-$data(t:y))*cos($az)+($x-$data(t:x))*sin($az)]
#	    set jZ [expr -($y-$data(t:y))*sin($az)+($x-$data(t:x))*cos($az)]
#	    set jX [expr $jX * 0.1*exp(sqrt($data(t:zoom))/10.0)]
#	    set jZ [expr $jZ * 0.1*exp(sqrt($data(t:zoom))/10.0)]
#		set jY [expr 90/3.141592654*atan2($jZ,abs($jX)) + $data(t:jY)]
#puts "jX = $jX -- jZ = $jZ   ===>  $jY"

		if {$data(t:axe)==""} {
			set delta1 [expr $x-$data(t:x)]
			set delta2 [expr $y-$data(t:y)]
			if {[expr abs($delta1)-abs($delta2)]<0} {
				set data(t:axe) y
			} else {
				set data(t:axe) x
			}
		} elseif {$data(t:axe)=="x"} {
			set jY [expr ($x-$data(t:x)) * exp(sqrt($data(t:zoom))/10.0) + $data(t:jY)]
		} else {
			set jY [expr ($y-$data(t:y)) * exp(sqrt($data(t:zoom))/10.0) + $data(t:jY)]
		}

		catch { $data(t:body) cset $data(t:joint):Y $jY }
	}
	return $data(t:ok)
}
