#include "togl.h"
#include <math.h>
#ifndef WIN32
#include <stdlib.h>		// for abs()
#endif
#include <GL/glu.h>

#include "list.h"
#include "view.h"
#include "routine.h"		// just for routine->number in .i

extern char *trackName[];

//================================================================================================

#include "viewTrack.icc"

//------------------------------------------------------------------------------------------------

ViewTrack::ViewTrack(Togl *t)
{
	togl = t;
	drawGrid = drawCurrentTime = drawRoutines = TRUE;
	drawHandles = FALSE;
	currentTime = -1;
	maxTime = maxY = 0;
	limitMin = -1000000; limitMax = 1000000;
	padTime = padY = 1000000;
	stippleTime  = 0xFFFF; stippleY  = 0x14C7;
	stippleTime2 = 0x0F0F; stippleY2 = 0x0101;
	stippleZoomBox = 0xF0F0;
	xPos = yPos = 0.0;
	xZoom = yZoom = 1.0;
	zbXmin = -1;	// no zoom box
	pointSize = 4.0;
	routine = NULL; dancer = -1;
	for(int i=0; i<MAX_VIEWTRACK; i++)
		track[i] = -1;
}

ViewTrack::~ViewTrack()
{
}

//------------------------------------------------------------------------------------------------

void ViewTrack::SetGLView(int picking, int x, int y) const
{
int width = Togl_Width(togl);
int height = Togl_Height(togl);
float aspect = (float) width / (float) height;
GLint viewport[4];


	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (picking) {
	    glGetIntegerv(GL_VIEWPORT, viewport);
	    gluPickMatrix((GLdouble) x, (GLdouble) (viewport[3] - y), 2.0, 2.0, viewport);
	}

ViewTrack *self = const_cast<ViewTrack *>(this); 
	self->xmin = (int) ((xZoom-1)*xPos*maxTime/xZoom);
	self->xmax = (int) ((1.0+(xZoom-1)*xPos)*maxTime/xZoom) + 1;
	self->ymin = (-yZoom+2*(yZoom-1)*(1-yPos))*maxY/yZoom;
	self->ymax = (-yZoom+2+2*(yZoom-1)*(1-yPos))*maxY/yZoom;
	glOrtho(xmin, xmax, ymin, ymax, 0.0, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glShadeModel(GL_FLAT);
}


void ViewTrack::Redisplay() const
{
int i;

	glClearColor(color.bg[0], color.bg[1], color.bg[2], color.bg[3]);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if (routine == NULL) {
		return;
	}

	SetGLView(FALSE, 0, 0);


	// the routine in background
	if (drawRoutines) {
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		for(ListIterator<Figure> f(routine->figures); f; ++f) {
			if (f()->start > xmax || f()->end < xmin) {
					continue;
			}

			for(int j=0; j<f()->nbMap; j++) {
				if (f()->map[j] == dancer) {
					for(i=0; i<MAX_VIEWTRACK; i++) {
						if (track[i]!=-1 && (f()->flags[j][track[i]] & USED)) {
							XColor xcolor;

							if (XParseColor(Tk_Display(Tk_MainWindow(togl->Interp)),
											Tk_Colormap(Tk_MainWindow(togl->Interp)),
							  				f()->color, &xcolor) != 0) {
								glColor3f(xcolor.red/65535.0, xcolor.green/65535.0,
										  xcolor.blue/65535.0);
								glRectd(f()->start, ymin, f()->end, ymax);
							}
						}
					}
				}
			}
		}
	}


	if (drawGrid) {
		glEnable(GL_LINE_STIPPLE);
		glColor3dv(color.grid);
		// time axis
		glLineStipple (1, stippleTime2);
		glBegin(GL_LINES);
		for(i=0; i <= maxTime; i += padTime2) {
			glVertex2i(i, -maxY); glVertex2i(i, maxY);
		}
		glEnd();
		glLineStipple (1, stippleTime);
		glBegin(GL_LINES);
		for(i=0; i <= maxTime; i += padTime) {
			glVertex2i(i, -maxY); glVertex2i(i, maxY);
		}
		glEnd();
		// value axis
		glLineStipple (1, stippleY2);
		glBegin(GL_LINES);
		for(i=0; i <= maxY; i += padY2) {
			glVertex2i(0, i);  glVertex2i(maxTime, i);
			glVertex2i(0, -i); glVertex2i(maxTime, -i);
		}
		glEnd();
		glLineStipple (1, stippleY);
		glBegin(GL_LINES);
		for(i=0; i <= maxY; i += padY) {
			glVertex2i(0, i);  glVertex2i(maxTime, i);
			glVertex2i(0, -i); glVertex2i(maxTime, -i);
		}
		glEnd();
		glDisable(GL_LINE_STIPPLE);
	}
	// main time axis
	glColor3dv(color.grid);
	glBegin(GL_LINES);
	  glVertex2i(0, 0);
	  glVertex2i(maxTime, 0);
	glEnd();
	// limits of the joints
	glColor3dv(color.limits);
	glBegin(GL_LINES);
	  glVertex2i(xmin, limitMin); glVertex2i(xmax, limitMin);
	  glVertex2i(xmin, limitMax); glVertex2i(xmax, limitMax);
	glEnd();

	// a line to symbolize the current time
	if (drawCurrentTime) {
		glColor3dv(color.time);
		glBegin(GL_LINES);
		  glVertex2i(currentTime, -maxY);
		  glVertex2i(currentTime, maxY);
		glEnd();
	}

	// the tracks
	for(i=0; i<MAX_VIEWTRACK; i++) {
		if (track[i] != -1) {
			ViewTrack::Draw(FORCE_WIRE, i);
		}
	}

	// the zoom box (if any)
	if (zbXmin != -1)  {
		glEnable(GL_LINE_STIPPLE);
		glColor3dv(color.zoomBox);
		glLineStipple (1, stippleZoomBox);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glRectd(zbXmin, zbYmin, zbXmax, zbYmax);
		glDisable(GL_LINE_STIPPLE);
	}				
}


int ViewTrack::Pick(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]) const
{
GLuint selectBuf[SELECT_BUFSIZE], *ptr, names, min = 0xffffffff;
GLint hits;
GLint viewport[4];
int x,y, i, trackId = -1, type = -1, time = -1;
Tcl_Obj *res;


	if (objc < 4) {
		Tcl_WrongNumArgs(interp, 2, objv, "x y");
		return TCL_ERROR;
	}

	if (routine == NULL) {
		Tcl_ResetResult(interp);
		return TCL_OK;
	}

	// get the coordinate of the pick
	Tcl_GetIntFromObj(interp, objv[2], &x);
	Tcl_GetIntFromObj(interp, objv[3], &y);

	// enter in selection mode
    glSelectBuffer(SELECT_BUFSIZE, selectBuf);
    glRenderMode(GL_SELECT);

    glInitNames();
    glPushName(0);

	SetGLView(TRUE, x, y);


	// draw the view to find the pick
	if (drawCurrentTime) {
		glLoadName(MAX_VIEWTRACK);	// handle for current time
		glPushName(0);
		glPushName(currentTime);
		glBegin(GL_LINES);
		  glVertex2i(currentTime, -maxY);
		  glVertex2i(currentTime, maxY);
		glEnd();
		glPopName();
		glPopName();
	}

	for(i=0; i<MAX_VIEWTRACK; i++) {
		if (track[i] != -1) {
			glLoadName(i);	// name of the track
			glPushName(0);
			ViewTrack::Draw(SELECT, i);
			glPopName();
		}
	}
    glFlush();
    hits = glRenderMode(GL_RENDER);

	// analyze the result
    ptr = selectBuf;
    for (i = 0; i < hits; i++) {
		if ((names = *ptr++) != 3) {
			Tcl_SetResult(interp, "[viewTrack::pick] internal error", NULL);
			return TCL_ERROR;
		}
		ptr++;	// skip the max
		ptr++;	// skip the min
		trackId = *ptr++;
		type = *ptr++;
		time = *ptr++;
	}

	// output result
	if (trackId >= 0 && trackId < MAX_VIEWTRACK) {
		res = Tcl_NewListObj(0, NULL);
		Tcl_ListObjAppendElement(interp, res,
				Tcl_NewStringObj((char *) (type == VT_LINE ? "line" :
										   type == VT_POINT ? "point" : "handle"), -1));
		Tcl_ListObjAppendElement(interp, res, Tcl_NewIntObj(trackId));
		Tcl_ListObjAppendElement(interp, res, Tcl_NewIntObj(time));
		Tcl_SetObjResult(interp, res);	
	} else if (trackId == MAX_VIEWTRACK) {
		res = Tcl_NewListObj(0, NULL);
		Tcl_ListObjAppendElement(interp, res, Tcl_NewStringObj("currenttime", -1));
		Tcl_ListObjAppendElement(interp, res, Tcl_NewIntObj(trackId));
		Tcl_ListObjAppendElement(interp, res, Tcl_NewIntObj(time));
		Tcl_SetObjResult(interp, res);	
	} else {
		Tcl_ResetResult(interp);
	}

	return TCL_OK;
}

int ViewTrack::PickFigure(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], int askID) const
{
int x, vx;

	if (objc < 3) {
		Tcl_WrongNumArgs(interp, 2, objv, "x");
		return TCL_ERROR;
	}

	if (routine == NULL) {
		Tcl_ResetResult(interp);
		return TCL_OK;
	}

	// get the coordinate of the pick
	Tcl_GetIntFromObj(interp, objv[2], &x);
	vx = (int) (xmin + (double) (xmax-xmin)*x/Togl_Width(togl));
	if (vx<0) vx=0;
	if (vx>maxTime) vx=maxTime;

	// get the routine name
	ListIterator<Figure> f;
	for(f.reset(routine->figures); f; ++f) {
		if (f()->start > xmax || f()->end < xmin) {
				continue;
		}

		for(int j=0; j<f()->nbMap; j++) {
			if (f()->map[j] == dancer) {
				if (f()->start <= vx  &&  vx <= f()->end)
					goto found; // inside 2 loops -> break is not working
			}
		}
	}

	found:
	if (f) {
		if (askID) {
			Tcl_SetObjResult(interp, Tcl_NewIntObj(f()->number));
		} else {
			Tcl_SetObjResult(interp, Tcl_NewStringObj(f()->name, -1));
		}
	} else {
		Tcl_ResetResult(interp);
	}

	return TCL_OK;
}


//------------------------------------------------------------------------------------------------

void ViewTrack::ScreenToView(int sx, int sy, int *vx, double *vy)
{
	*vx = (int) (xmin + (double) (xmax-xmin)*sx/Togl_Width(togl));
	if (*vx<0) *vx=0;
	if (*vx>maxTime) *vx=maxTime;
	*vy = (ymin + (ymax-ymin)*(Togl_Height(togl)-sy)/Togl_Height(togl));
	if (*vy<-maxY) *vy=-maxY;
	if (*vy>maxY) *vy=maxY;
}

int ViewTrack::Convert(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
{
int x,y, vx;
double vy;
Tcl_Obj *res;

	if (objc < 4) {
		Tcl_WrongNumArgs(interp, 2, objv, "x y");
		return TCL_ERROR;
	}

	// get the coordinate of the pick
	Tcl_GetIntFromObj(interp, objv[2], &x);
	Tcl_GetIntFromObj(interp, objv[3], &y);

	// convert from screen to view
	ScreenToView(x,y, &vx,&vy);

	res = Tcl_NewListObj(0, NULL);
	Tcl_ListObjAppendElement(interp, res, Tcl_NewIntObj(vx));
	Tcl_ListObjAppendElement(interp, res, Tcl_NewDoubleObj(((int) (vy*100))/100.0));

	Tcl_SetObjResult(interp, res);	
	return TCL_OK;
}

//------------------------------------------------------------------------------------------------

int ViewTrack::Limits(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
{
int nb;
Tcl_Obj **list;

	if (objc != 3 ) {
		Tcl_WrongNumArgs(interp, 2, objv, "{min max}");
		return TCL_ERROR;
	}

	Tcl_ListObjGetElements(interp, objv[2], &nb, &list);
	if (nb != 2) {
		Tcl_WrongNumArgs(interp, 2, objv, "{min max}");
		return TCL_ERROR;
	}
	Tcl_GetIntFromObj(NULL, list[0], &limitMin);
	Tcl_GetIntFromObj(NULL, list[1], &limitMax);

	Togl_PostRedisplay(togl);
	return TCL_OK;
}

//------------------------------------------------------------------------------------------------

int ViewTrack::Move(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
{
static char *typePtr[] = {"point", "handle", "handlelock", "handletime", (char *) NULL};
int index;
int x,y, vx, trackId, time;
double vy;


	if (objc < 5) {
		Tcl_WrongNumArgs(interp, 2, objv, "track type time ?newtime? value");
		return TCL_ERROR;
	}

	// track
	Tcl_GetIntFromObj(interp, objv[2], &trackId);
	if (trackId<0 || trackId >= MAX_VIEWTRACK) {
		Tcl_SetResult(interp, "bad track index", NULL);
		return TCL_ERROR;
	}

	// type
	if (Tcl_GetIndexFromObj(interp, objv[3], typePtr, "type", 0, &index) != TCL_OK) {
		return TCL_ERROR;
	}

	// get the coordinate of the pick
	if (objc == 6) {
		x = -1;
		Tcl_GetIntFromObj(interp, objv[5], &y);
	} else {
		Tcl_GetIntFromObj(interp, objv[5], &x);
		Tcl_GetIntFromObj(interp, objv[6], &y);
	}
	// convert from screen to view
	ScreenToView(x,y, &vx,&vy);

	// reference of the point to move
	Tcl_GetIntFromObj(interp, objv[4], &time);

	// tell the routine to perform the change and record it
	routine->ModifyValue(time, dancer, track[trackId], index, vx, vy);

	Togl_PostRedisplay(togl);
	return TCL_OK;
}

//------------------------------------------------------------------------------------------------

int ViewTrack::Modify(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
{
static char *typePtr[] = {"linearizeTime", "linearizeValue",
						  "linearizeAll", "smoothAll", (char *) NULL};
int index;
int trackId, time;

	if (objc < 5) {
		Tcl_WrongNumArgs(interp, 2, objv, "track type time");
		return TCL_ERROR;
	}

	// track
	Tcl_GetIntFromObj(interp, objv[2], &trackId);
	if (trackId<0 || trackId >= MAX_VIEWTRACK) {
		Tcl_SetResult(interp, "bad track index", NULL);
		return TCL_ERROR;
	}

	// type
	if (Tcl_GetIndexFromObj(interp, objv[3], typePtr, "type", 0, &index) != TCL_OK) {
		return TCL_ERROR;
	}

	// reference of the point to move
	Tcl_GetIntFromObj(interp, objv[4], &time);

	// tell the routine to perform the change and record it
	routine->ModifyType(time, dancer, track[trackId], index);

	Togl_PostRedisplay(togl);
	return TCL_OK;
}

//------------------------------------------------------------------------------------------------

int ViewTrack::Zoom(Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
{
static char *cmdPtr[] = {"start", "drag", "set", (char *) NULL};
enum {start, drag, set} index;
int x,y, vx;
double vy;


	if (objc < 5) {
		Tcl_WrongNumArgs(interp, 2, objv, "command x y");
		return TCL_ERROR;
	}

	if (Tcl_GetIndexFromObj(interp, objv[2], cmdPtr, "command", 0, (int *) &index) != TCL_OK) {
		return TCL_ERROR;
	}

	// get the coordinate of the pick
	Tcl_GetIntFromObj(interp, objv[3], &x);
	Tcl_GetIntFromObj(interp, objv[4], &y);

	// convert coords
	ScreenToView(x,y, &vx,&vy);
	
	switch (index) {
		case start:		zbXmin = zbXmax = vx;
						zbYmin = zbYmax = vy;
						break;

		case drag:		zbXmax = vx;
						zbYmax = vy;
						break;

		case set:		if (abs(zbXmin-zbXmax)>2 && fabs(zbYmin-zbYmax)>2) {
							// compute new settings
							xZoom = maxTime/fabs(zbXmin-zbXmax);
							if (zbXmax<zbXmin) zbXmin=zbXmax;
							xPos = zbXmin*xZoom/((xZoom-1)*maxTime);

							yZoom = 2.0*maxY/fabs(zbYmin-zbYmax);
							if (zbYmax<zbYmin) zbYmin=zbYmax;
							yPos = 1.0-((zbYmin*yZoom/maxY)+yZoom)/(2*(yZoom-1));
						}
						// reset the drag box
						zbXmin = -1;
						break;
	}

	Togl_PostRedisplay(togl);
	return TCL_OK;
}


//================================================================================================


int ViewTrack_ObjCmd(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
{
struct Togl *togl = (struct Togl *)clientData;
struct ViewTrack *view = (struct ViewTrack *) Togl_GetClientData(togl);
static char *optionsPtr[] = {
	"configure", "render", "swapbuffers", "makecurrent", 
	"get", "set", "convert", "pick", "pickfigure", "pickfigureid", "zoom",
	"limits", "move", "modify", "insert", "delete", (char *) NULL};
enum {
	config, render, swap, current,
	get, set, convert, pick, pickfig, pickfigID, zoom,
	limits, move, modify, insert, del } index;


	if (objc < 2) {
		Tcl_WrongNumArgs(interp, 1, objv, "?options?");
		return TCL_ERROR;
	}

	if (Tcl_GetIndexFromObj(interp, objv[1], optionsPtr, "option", 0, (int *) &index) != TCL_OK) {
		return TCL_ERROR;
	}

	Togl_MakeCurrent(togl);

	switch (index) {
		case config:
		case render:
		case swap:
		case current:	return Togl_DefaultProcessing(togl, interp, index, objc, objv);

		case get:		return view->Get(interp, objc, objv);
		case set:		return view->Set(interp, objc, objv);
		case convert:	return view->Convert(interp, objc, objv);
		case pick:		return view->Pick(interp, objc, objv);
		case pickfig:	
		case pickfigID:	return view->PickFigure(interp, objc, objv,
												index == pickfigID ? TRUE : FALSE);

		case limits:	return view->Limits(interp, objc, objv);
		case move:		return view->Move(interp, objc, objv);
		case zoom:		return view->Zoom(interp, objc, objv);
		case modify:	return view->Modify(interp, objc, objv);
//		case insert:	return view->Pick(interp, objc, objv);
//		case del:		return view->Pick(interp, objc, objv);
	}

	return TCL_OK;
}
