namespace eval viewTrack {
	variable list {}

	image create pixmap zoomIn -file pixmap/zoomin.xpm
	image create pixmap zoomOut -file pixmap/zoomout.xpm
	image create pixmap raz -file pixmap/raz.xpm

	image create pixmap tbt_grid -file pixmap/tb_track_grid.xpm
	image create pixmap tbt_rtn -file pixmap/tb_track_routine.xpm
	image create pixmap tbt_time -file pixmap/tb_track_time.xpm
	image create pixmap tbt_hnd -file pixmap/tb_track_handle.xpm
}

array set viewTrack::categories {
	"cog"		{{0 position} {0.0 X "cog:X"} {0.1 Y "cog:Y"} {0.2 Z "cog:Z"}
				 {1 angle} {1.0 X "cog:rX"} {1.1 Y "cog:rY"} {1.2 Z "cog:rZ"} }
	"body"		{{0 torso} {0.0 X "torso:X"} {0.1 Y "torso:Y"} {0.2 Z "torso:Z"}
				 {1 pelvis} {1.0 X "pelvis:X"} {1.1 Y "pelvis:Y"} {1.2 Z "pelvis:Z"}
				 {2 head} {2.0 X "head:X"} {2.1 Y "head:Y"} {2.2 Z "head:Z"} }

	"left arm"	{{0 shoulder} {0.0 X "left:shoulder:X"} {0.1 Y "left:shoulder:Y"}
							  {0.2 Z "left:shoulder:Z"} {0.3 trans "left:shoulder:TY"}
				 {1 elbow} {1.0 X "left:elbow:X"} {1.1 Y "left:elbow:Y"} 
				 {2 wrist} {2.0 X "left:wrist:X"} {2.1 Y "left:wrist:Y"} {2.2 Z "left:wrist:Z"} }
	"right arm"	{{0 shoulder} {0.0 X "right:shoulder:X"} {0.1 Y "right:shoulder:Y"}
							  {0.2 Z "right:shoulder:Z"} {0.3 trans "right:shoulder:TY"}
				 {1 elbow} {1.0 X "right:elbow:X"} {1.1 Y "right:elbow:Y"} 
				 {2 wrist} {2.0 X "right:wrist:X"} {2.1 Y "right:wrist:Y"}
						   {2.2 Z "right:wrist:Z"} }

	"left leg"	{{0 hip} {0.0 X "left:hip:X"} {0.1 Y "left:hip:Y"} {0.2 Z "left:hip:Z"}
				 {1 knee} {1.0 X "left:knee:X"}
				 {2 ankle} {2.0 X "left:ankle:X"} {2.1 Y "left:ankle:Y"} {2.2 Z "left:ankle:Z"}
				 {3 foot} {3.0 X "left:foot:X"} }
	"right leg"	{{0 hip} {0.0 X "right:hip:X"} {0.1 Y "right:hip:Y"} {0.2 Z "right:hip:Z"}
				 {1 knee} {1.0 X "right:knee:X"}
				 {2 ankle} {2.0 X "right:ankle:X"} {2.1 Y "right:ankle:Y"}
						   {2.2 Z "right:ankle:Z"}
				 {3 foot} {3.0 X "right:foot:X"} }
}

if {$tcl_platform(platform) == "windows"} {
	array set viewTrack::accelerator {
		"cog"			{2	2}
		"body"			{5	5}
		"left arm"		{4	4}
		"left leg"		{1	1}
		"right arm"		{6	6}
		"right leg"		{3	3}
	}
} else {
	array set viewTrack::accelerator {
		"cog"			{KP_Down	2}
		"body"			{KP_Begin	5}
		"left arm"		{KP_Left	4}
		"left leg"		{KP_End		1}
		"right arm"		{KP_Right	6}
		"right leg"		{KP_Next	3}
	}
}


proc viewTrack::category {w class} {
variable $w
upvar 0 $w data
variable categories
variable name


	# save parameters for the view and restore old one for the class
	set data(v:view:$data(v:class)) [$data(w:view) get view]
	$data(w:view) set view $data(v:view:$class)
	set data(v:class) $class
	  # x axis
	  set zoom [$data(w:view) get xzoom]
	  set pos [expr [$data(w:view) get xpos]*($zoom-1)/$zoom]
	  scroll:CB $data(w:view) $data(w:sx) x moveto $pos
	  # y axis
	  set zoom [$data(w:view) get yzoom]
	  set pos [expr [$data(w:view) get ypos]*($zoom-1)/$zoom]
	  scroll:CB $data(w:view) $data(w:sy) y moveto $pos

	# update select box
	set list [$data(w:select) subwidget hlist]
	$list delete all
 
	foreach i {0 1 2 3 4 5 6 7 8 9} {
		$data(w:view) set track $i nil
	}

	set i 0
	foreach entry $categories($class) {
		$list add [lindex $entry 0] -text [lindex $entry 1]
		if {[lindex $entry 2] != {}} {
			set name([lindex $entry 0]) [list $i [lindex $entry 2]]
			incr i
		} else {
			set name([lindex $entry 0]) {-1 -1}
		}
		$data(w:select) setstatus [lindex $entry 0] on
	}

	$data(w:select) autosetmode
}


array set viewTrack::subcategories {
	"cog:p"	{on off}			"cog:a"	{off on}			"cog:*" {on on}
	"body:t" {on off off}		"body:p" {off on off}		"body:h" {off off on}
	"left arm:s" {on off off}	"left arm:e" {off on off}	"left arm:w" {off off on}
	"right arm:s" {on off off}	"right arm:e" {off on off}	"right arm:w" {off off on}
	"left leg:h" {on off off off}	"left leg:k" {off on off off}
	"left leg:a" {off off on off}	"left leg:f" {off off off on}
	"right leg:h" {on off off off}	"right leg:k" {off on off off}
	"right leg:a" {off off on off}	"right leg:f" {off off off on}
}

proc viewTrack::subcategory {top shortcut} {
variable subcategories

	set what "[set viewTrack::${top}(v:class)]:$shortcut"
	catch {
		set i 0
		foreach sel $subcategories($what) {
			$top.select setstatus $i $sel
			incr i
		}
	}
}

array set viewTrack::part2category {
	"cog" {cog *}
	"torso" {body t} "pelvis" {body p} "neck" {body h} "head" {body h}

	"left:hip" {"left leg" h} "left:knee" {"left leg" k} 
	"left:ankle" {"left leg" a} "left:foot" {"left leg" f}
	"left:shoulder" {"left arm" s} "left:elbow" {"left arm" e} "left:wrist" {"left arm" w}

	"right:hip" {"right leg" h} "right:knee" {"right leg" k} 
	"right:ankle" {"right leg" a} "right:foot" {"right leg" f}
	"right:shoulder" {"right arm" s} "right:elbow" {"right arm" e} "right:wrist" {"right arm" w}
}


proc viewTrack::select {w entry mode} {
variable name

	set data $name($entry)
	if {[lindex $data 0] == -1} {
		return
	}
	if {$mode == "on"} {
		$w set track [lindex $data 0] [lindex $data 1]
	} elseif {$mode == "off"} {
		$w set track [lindex $data 0] nil
	} 
}


#=================================================================================================


proc viewTrack::createView {w routine} {
variable $w
upvar 0 $w data
variable categories
variable accelerator

	set t [frame $w.t]

	#---- the viewport ----
	# the view
	set data(w:view) [togl $t.view -width 400 -height 220 -ident "$w" \
			-class viewTrack -double true -depth true]
	bind $t.view <ButtonPress-2> "viewTrack::scroll:start $w %x %y"
	bind $t.view <Button2-Motion> "viewTrack::scroll:drag $w %x %y"
	bind $t.view <Any-Motion> "viewTrack::motion $w %x %y"
	bind $t.view <ButtonPress-1> "viewTrack::move:setup $w %x %y"
	bind $t.view <Button1-Motion> "viewTrack::move:drag $w %x %y"
	bind $t.view <ButtonRelease-1> "viewTrack::move:release $w %x %y"

	helpCreate $t.view viewTrack::${w}(st:help) viewTrack::help viewTrack::${w}(v:mode)

	# the X scrollbar + zoom buttons
	set data(w:sx) [scrollbar $t.sx -orient horizontal \
			-command "viewTrack::scroll:CB $t.view $t.sx x"]
	button $t.zxIn  -bd 1 -padx 0 -pady 0 -image zoomIn \
					-command "viewTrack::updateZoom xzoom +1 $t.view $t.sx"
	button $t.zxOut -bd 1 -padx 0 -pady 0 -image zoomOut \
					-command "viewTrack::updateZoom xzoom -1 $t.view $t.sx"
	# the Y scrollbar + zoom buttons
	set data(w:sy) [scrollbar $t.sy -orient vertical \
			-command "viewTrack::scroll:CB $t.view $t.sy y"]
	button $t.zyIn  -bd 1 -padx 0 -pady 0 -image zoomIn \
					-command "viewTrack::updateZoom yzoom +1 $t.view $t.sy"
	button $t.zyOut -bd 1 -padx 0 -pady 0 -image zoomOut \
					-command "viewTrack::updateZoom yzoom -1 $t.view $t.sy"
	# the reset button
	button $t.raz   -bd 1 -image raz -command "$t.view set xzoom 1; $t.sx set 0 1; \
			$t.view set pady {10 45}; $t.view set maxy 180; $t.view set yzoom 1; $t.sy set 0 1"

	# the layout of the viewing
	grid $t.view	-		-		$t.zyOut -sticky news
	grid ^			^		^		$t.zyIn	 -sticky news
	grid ^			^		^		$t.sy	 -sticky news
	grid $t.zxOut	$t.zxIn $t.sx	$t.raz	 -sticky news
	grid rowconfig    $t {0 1 3 4} -weight 0
	grid rowconfig    $t 2 -weight 1
	grid columnconfig $t {0 1 3} -weight 0
	grid columnconfig $t 2 -weight 1

	#---- the controls ----
	# the toolbar
	set data(w:tb) [toolbar::new $w.toolbar -bd 1]
	set tb $data(w:tb)
	  toolbar::add:check $tb grid -bd 1 -image tbt_grid -tip "Show the grid" \
			-oncommand "$data(w:view) set showgrid 1" \
			-offcommand "$data(w:view) set showgrid 0" 
	  toolbar::add:check $tb routine -bd 1 -image tbt_rtn -tip "Show routines" \
			-oncommand "$data(w:view) set showroutines 1" \
			-offcommand "$data(w:view) set showroutines 0" 
	  toolbar::add:check $tb time -bd 1 -image tbt_time -tip "Show time" \
			-oncommand "$data(w:view) set showtime 1" \
			-offcommand "$data(w:view) set showtime 0" 
	  toolbar::add:check $tb handles -bd 1 -image tbt_hnd -tip "Show handles of curves" \
			-oncommand "$data(w:view) set showhandles 1" \
			-offcommand "$data(w:view) set showhandles 0" 

	  toolbar::vertical $tb

 	  toolbar::check:setOn $tb grid
 	  toolbar::check:setOn $tb routine
 	  toolbar::check:setOn $tb time
 
	# the status labels
	set s [frame $w.status]
	label $s.coord -relief sunken -bd 1 -font status -width 13 \
			-textvariable viewTrack::${w}(v:coord)
	label $s.track -relief sunken -bd 1 -font status -width 14 \
			-textvariable viewTrack::${w}(v:track)
	label $s.figname -relief sunken -bd 1 -font status -width 20 \
			-textvariable viewTrack::${w}(v:figname)
	label $s.help -relief sunken -bd 1 -font status -width 1 \
			-textvariable viewTrack::${w}(st:help)
	grid $s.coord $s.track $s.figname $s.help -sticky news
	grid columnconfig $s {3} -weight 1
	grid columnconfig $s {0 1 2} -weight 0
	
	# the class of joints representation
	menubutton $w.class -indicatoron 1 -menu $w.class.menu \
			-textvariable viewTrack::${w}(v:class) -relief raised \
			-bd 1 -padx 3 -pady 3 -anchor w -font menubutton:small -direction below
	set m [menu $w.class.menu -tearoff 0 -font menubutton:small -activeborderwidth 1 -bd 1]

	foreach cat {"cog" "body" "left arm" "right arm" "left leg"	"right leg"} {
		$m add command -label "$cat         " -accelerator [lindex $accelerator($cat) 1] \
				-command "viewTrack::category $w \"$cat\""
		bind $w <Key-[lindex $accelerator($cat) 0]> "viewTrack::category $w \"$cat\""
		bind $w <Key-[lindex $accelerator($cat) 1]> "viewTrack::category $w \"$cat\""
	}
	foreach shortcut {t p h k a e s w f} {
		bind $w $shortcut "viewTrack::subcategory $w %K"
	}

	# the hlist to select them
    tixAutoCheckList $w.select -callback "viewTrack::select $t.view" \
			-highlightthickness	0 \
			-options {	hlist.font		viewtrack
						hlist.padY		2
						hlist.width		12 }
	set data(w:select) $w.select

	grid $tb	$t	$w.class	-sticky news
	grid x		x	$w.select	-sticky news
	grid $s		-	-			-sticky news -padx 1 -pady 1

	grid configure $tb -rowspan 2
	grid configure $t -rowspan 2
	grid rowconfig    $w {0 2} -weight 0
	grid rowconfig    $w {1} -weight 1
	grid columnconfig $w {1} -weight 1
	grid columnconfig $w {0 2} -weight 0

	# some init
	$t.view set colorLine 0 black
	$t.view set colorLine 1 royalblue
	$t.view set colorLine 2 sienna
	$t.view set colorLine 3 midnightblue
	$t.view set colorLine 4 seagreen4
	$t.view set colorLine 5 firebrick
	$t.view set colorLine 6 darkorange
	$t.view set colorLine 7 maroon
	$t.view set colorLine 8 darkslategray
	$t.view set colorLine 9 blue
	$t.view set colorBg gray92
	$t.view set colorHandle seagreen
	$t.view set colorTime violet
	$t.sx set 0 1
	$t.sy set 0 1

	$t.view set maxtime [$routine get beats]
	$t.view set maxy 180

	$t.view set padtime {500 1000}
	$t.view set pady {10 45}

	foreach class [array names categories] {
		set data(v:view:$class) [$t.view get view]
	}
	category $w cog

	$t.view set time [$routine get time]	

	set data(v:showroutines) 1
}


proc viewTrack::new {w routineEditor body {part {}} {pos ""} {size ""}} {
variable part2category
variable list
variable $w
upvar 0 $w data

	if {[winfo exists $w]} {
		raise $w
		if {$part != {}} {
			set entry $part2category($part)
			category $w [lindex $entry 0]
			subcategory $w [lindex $entry 1]
		}
		return
	}

	# register the editor
	lappend list $w

	# create the window
    set data(w:top) $w
    toplevel $w
	if {$pos != "" || $size != ""} {
	    wm geometry $w $size$pos
	}
    wm title $w "Track Editor <[$body get name]>"
	wm protocol $w WM_DELETE_WINDOW "viewTrack::close $w"
	bind $w <q> "after idle \"viewTrack::close $w\""
	bind $w <Alt-w> "after idle \"viewTrack::close $w\""
	bind $w <Escape> "after idle \"viewTrack::close $w\""

	# find which dancer
	set data(v:routineEditor) $routineEditor
	set routine [routine::getRoutine $routineEditor]
	viewTrack::createView $w $routine
	for {set i 0} {$i < [$routine get dancers]} {incr i} {
		if ![string compare [$routine get body $i] $body] {
			$data(w:view) set dancer $routine $i
			break;
		}
	}

	# bindings
	bindtags $w.t.view "[bindtags $w.t.view] KeyRoutine"

	if {$part != {}} {
		set entry $part2category($part)
		category $w [lindex $entry 0]
		subcategory $w [lindex $entry 1]
	}

	# action menu
	menu $w.ma -tearoff 0 -font menubutton -bd 1 -activeborderwidth 1
	  $w.ma add command -label "Linearize"
	  $w.ma add command -label "Linearize time"
	  $w.ma add command -label "Linearize value"
	  $w.ma add separator
	  $w.ma add command -label "Linearize track"
	  $w.ma add command -label "Smooth track"
	bind $data(w:view) <3> "viewTrack::actionMenu $w %x %y %X %Y"
}

proc viewTrack::close {w} {
variable list
variable $w
upvar 0 $w data

	set i [lsearch -exact $list $w]
	set list [lreplace $list $i $i]

	destroy $w
	# remove array associated with window
	catch {unset data}
}

proc viewTrack::setTime {w t} {
variable $w
upvar 0 $w data

	$data(w:view) set time $t	
}

proc viewTrack::setMaxTime {w t} {
variable $w
upvar 0 $w data

	$data(w:view) set maxtime $t	
}

proc viewTrack::redisplay {w} {
variable $w
upvar 0 $w data

	$data(w:view) render	
}

#=================================================================================================

proc viewTrack::updateZoom {var amount view scroll} {
	set oldzoom [$view get $var]
	$view set $var [expr $oldzoom+$amount]
	if {$var == "yzoom"} {
		if {[expr $oldzoom+$amount] < 1.0} {
			$view set maxy [expr [$view get maxy] + [$view get maxy]/2]
			set maxy [$view get maxy]
			$view set pady [list [expr $maxy/10] [expr $maxy/4]]
		}
		set pos [expr .5-.5/[$view get yzoom]]
	} else {
		set pos [lindex [$scroll get] 0]
	}
	scroll:CB $view $scroll [string range $var 0 0] moveto $pos
}

#-------------------------------------------------------------------------------------------------

proc viewTrack::scroll:CB {w scroll axe cmd args} {
	set zoom [$w get ${axe}zoom]
	if {$zoom == 1} {
		$scroll set 0 1
		return
	}
	set pos ${axe}pos

	# kind of scrolling
 	if {$cmd == "scroll"} {
		if {[lindex $args 1] == "units"} {
			set args [expr [$w get $pos]*($zoom-1)/$zoom + [lindex $args 0]*0.25/$zoom]
		} else {
			set args [expr [$w get $pos]*($zoom-1)/$zoom + [lindex $args 0]*1.00/$zoom]
		}
	}

	# force position between 0 and 1
	if {$args<0} {set args 0}
	if {$args>($zoom-1)/$zoom} {set args [expr ($zoom-1)/$zoom]}
	# update the view and the scrollbar
	$w set $pos [expr $zoom*$args/($zoom-1)]
	$scroll set $args [expr $args+1/$zoom]
}

proc viewTrack::scroll:start {w x y} {
variable $w
upvar 0 $w data

	set data(t:x) $x
	set data(t:y) $y
	set data(t:init:sx) [lindex [$data(w:sx) get] 0]
	set data(t:init:sy) [lindex [$data(w:sy) get] 0]
}

proc viewTrack::scroll:drag {w x y} {
variable $w
upvar 0 $w data

	set deltaX [$data(w:sx) delta [expr -$x+$data(t:x)] 0]
	set deltaY [$data(w:sy) delta 0 [expr -$y+$data(t:y)]]

	scroll:CB $data(w:view) $data(w:sx) x moveto [expr $data(t:init:sx)+$deltaX]
	scroll:CB $data(w:view) $data(w:sy) y moveto [expr $data(t:init:sy)+$deltaY]
}

#-------------------------------------------------------------------------------------------------

proc viewTrack::move:setup {w x y} {
variable $w
upvar 0 $w data

	set res [$data(w:view) pick $x $y]
	if {$res != ""} {
		if {[lindex $res 0] != "line"} {
			# check if in figure. If true, then check the existence of the red point
			if {[lindex $res 0] != "currenttime"} {
				routine::canvas:checkModif $data(v:routineEditor) \
						[lindex [$data(w:view) get dancer] 1] \
						[$data(w:view) get track [lindex $res 1]] \
						[lindex $res 2] \
						[$data(w:view) pickfigureid $x]
			}
			# some init
			set data(t:type) [lindex $res 0]
			set data(t:track) [lindex $res 1]
			set data(t:time) [lindex $res 2]
			move:drag $w $x $y
			if {[lindex $res 0] != "currenttime"} {
				$data(w:view) limits [body0 limits $data(v:track)]
			}
		}
	} else {
		set data(t:type) "zoom"
		$data(w:view) zoom start $x $y
	}
}

proc viewTrack::move:drag {w x y} {
variable $w
upvar 0 $w data

	set data(v:coord) [$data(w:view) convert $x $y]

	# try to change the time
	if {$data(t:type) == "currenttime"} {
		routine::sync $data(v:routineEditor) [lindex $data(v:coord) 0]
		return
	}

	# user is zooming an area
	if {$data(t:type) == "zoom"} {
		$data(w:view) zoom drag $x $y
		return
	}

	# move an object
	if {$data(t:type) == "point"} {
		$data(w:view) move $data(t:track) point $data(t:time) $y
	} elseif {$data(t:type) == "handle"} {
		if {$data(v:mode) == "shift"} {
			$data(w:view) move $data(t:track) handlelock $data(t:time) $x $y
		} elseif {$data(v:mode) == "control"} {
			$data(w:view) move $data(t:track) handletime $data(t:time) $x $y
		} else {
			$data(w:view) move $data(t:track) handle $data(t:time) $x $y
		}
	}

	set tmp [$data(w:view) get dancer]
	# routine       sync      current time
	#			dancer                joint
	[lindex $tmp 0] sync [$data(w:view) get time] \
			[lindex $tmp 1] [$data(w:view) get track $data(t:track)]
	if {$data(v:class) == "cog"} {
		foreach st $stage::list {
			stage::updateDisplay $st
		}
	}
}

proc viewTrack::move:release {w x y} {
variable $w
upvar 0 $w data

	if {$data(t:type) == "zoom"} {
		$data(w:view) zoom set $x $y
		scroll:CB $data(w:view) $data(w:sx) x scroll 0 units
		scroll:CB $data(w:view) $data(w:sy) y scroll 0 units
	} else {
		$data(w:view) limits {-1000000 1000000}
	}
}

#-------------------------------------------------------------------------------------------------

proc viewTrack::motion {w x y} {
variable $w
upvar 0 $w data

	set data(v:coord) [$data(w:view) convert $x $y]

	set res [$data(w:view) pick $x $y]
	set data(v:figname) [$data(w:view) pickfigure $x]

	if {$res != ""} {
		if {[lindex $res 0] == "currenttime"} {
			$data(w:view) configure -cursor sb_h_double_arrow
			set data(v:track) {}
			return
		}

		set data(v:track) [$data(w:view) get track [lindex $res 1]]
		if {[lindex $res 0] == "handle"} {
			if {$data(v:mode) == "control"} {
				$data(w:view) configure -cursor fleur
			} else {
				$data(w:view) configure -cursor sb_v_double_arrow
			}
		} elseif {[lindex $res 0] == "point"} {
			$data(w:view) configure -cursor sb_v_double_arrow
		} elseif {[lindex $res 0] == "line"} {
			$data(w:view) configure -cursor {}
		}
	} else {
		set data(v:track) {}
		$data(w:view) configure -cursor {}
	}
} 

#=================================================================================================

proc viewTrack::actionMenu {w x y X Y} {
variable $w
upvar 0 $w data

	if {$data(v:track) != ""} {
		# action on a track
		set res [$data(w:view) pick $x $y]
		set time [lindex $res 2]
		set track [lindex $res 1]

		$w.ma entryconfigure 0 -command "$data(w:view) modify $track linearizeTime $time; \
										 $data(w:view) modify $track linearizeValue $time;"
		$w.ma entryconfigure 1 -command "$data(w:view) modify $track linearizeTime $time"
		$w.ma entryconfigure 2 -command "$data(w:view) modify $track linearizeValue $time"

		$w.ma entryconfigure 4 -command "$data(w:view) modify $track linearizeAll $time"
		$w.ma entryconfigure 5 -command "$data(w:view) modify $track smoothAll $time"

		tk_popup $w.ma $X $Y
	}
}

#=================================================================================================

array set viewTrack::help {
	normal	"<1> Move object   <2> Scroll view   <3> Action menu"
	shift	"<1> Move object (both handles)  <2> Scroll view   <3> Action menu"
	control	"<1> Move object   <2> Scroll view   <3> Action menu"
	alt		"<1> Move object   <2> Scroll view   <3> Action menu"
	off		""
}
